# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class SetupEmail(Base):

    _name_locator = (By.CSS_SELECTOR, 'section.card-setup-account-info input.sup-info-name')
    _email_locator = (By.CSS_SELECTOR, 'section.card-setup-account-info input.sup-info-email')
    _password_locator = (By.CSS_SELECTOR, 'section.card-setup-account-info input.sup-info-password')
    _next_locator = (By.CSS_SELECTOR, '.sup-info-next-btn')
    _continue_button_locator = ('class name', 'sup-show-mail-btn sup-form-btn recommend')
    _check_for_new_messages_locator = (By.CSS_SELECTOR, '.tng-account-check-interval.mail-select')
    _account_prefs_next_locator = (By.CSS_SELECTOR, '.card-setup-account-prefs .sup-info-next-btn')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._name_locator)

    def type_name(self, value):
        self.marionette.find_element(*self._name_locator).send_keys(value)

    def type_email(self, value):
        self.marionette.find_element(*self._email_locator).send_keys(value)

    def type_password(self, value):
        self.marionette.find_element(*self._password_locator).send_keys(value)

    def tap_next(self):
        # TODO Tap one pixel above bottom edge to dodge the System update notification banner bug 879192
        next_button = self.marionette.find_element(*self._next_locator)
        next_button.tap(y=(next_button.size['height'] - 1))
        self.wait_for_element_displayed(*self._check_for_new_messages_locator)

    def tap_account_prefs_next(self):
        self.marionette.find_element(*self._account_prefs_next_locator).tap()

    def wait_for_setup_complete(self):
        self.wait_for_element_displayed(*self._continue_button_locator)

    def tap_continue(self):
        self.marionette.find_element(*self._continue_button_locator).tap()


class ManualSetupEmail(Base):

    name = 'E-Mail'  # hack to be able to use select

    _name_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config input.sup-info-name')
    _email_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config input.sup-info-email')
    _password_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config input.sup-info-password')

    _account_type_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-account-type')

    _imap_username_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-composite-username')
    _imap_hostname_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-composite-hostname')
    _imap_port_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-composite-port')

    _smtp_username_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-smtp-username')
    _smtp_hostname_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-smtp-hostname')
    _smtp_port_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-smtp-port')

    _activesync_hostname_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-activesync-hostname')
    _activesync_username_locator = (By.CSS_SELECTOR, 'section.card-setup-manual-config .sup-manual-activesync-username')

    _next_locator = (By.CSS_SELECTOR, '.sup-manual-next-btn')
    _continue_button_locator = (By.CLASS_NAME, 'sup-show-mail-btn sup-form-btn recommend')

    _check_for_new_messages_locator = (By.CSS_SELECTOR, '.tng-account-check-interval.mail-select')
    _account_prefs_next_locator = (By.CSS_SELECTOR, '.card-setup-account-prefs .sup-info-next-btn')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._name_locator)

    def type_name(self, value):
        el = self.marionette.find_element(*self._name_locator)
        el.clear()
        el.send_keys(value)

    def type_email(self, value):
        el = self.marionette.find_element(*self._email_locator)
        el.clear()
        el.send_keys(value)
        self.keyboard.dismiss()

    def type_password(self, value):
        el = self.marionette.find_element(*self._password_locator)
        el.clear()
        el.send_keys(value)
        self.keyboard.dismiss()

    def select_account_type(self, value):
        account_type = self.marionette.find_element(*self._account_type_locator)
        # TODO: remove the explicit scroll once bug 833370 is fixed
        self.marionette.execute_script("arguments[0].scrollIntoView(false);", [account_type])
        account_type.click()
        self.select(value)

    def type_imap_name(self, value):
        el = self.marionette.find_element(*self._imap_username_locator)
        el.clear()
        el.send_keys(value)

    def type_imap_hostname(self, value):
        el = self.marionette.find_element(*self._imap_hostname_locator)
        el.clear()
        el.send_keys(value)

    def type_imap_port(self, value):
        el = self.marionette.find_element(*self._imap_port_locator)
        el.clear()
        el.send_keys(value)

    def type_smtp_name(self, value):
        el = self.marionette.find_element(*self._smtp_username_locator)
        el.clear()
        el.send_keys(value)

    def type_smtp_hostname(self, value):
        el = self.marionette.find_element(*self._smtp_hostname_locator)
        el.clear()
        el.send_keys(value)

    def type_smtp_port(self, value):
        el = self.marionette.find_element(*self._smtp_port_locator)
        el.clear()
        el.send_keys(value)

    def type_activesync_name(self, value):
        el = self.marionette.find_element(*self._activesync_username_locator)
        el.clear()
        el.send_keys(value)

    def type_activesync_hostname(self, value):
        el = self.marionette.find_element(*self._activesync_hostname_locator)
        el.clear()
        el.send_keys(value)

    def type_activesync_port(self, value):
        el = self.marionette.find_element(*self._activesync_port_locator)
        el.clear()
        el.send_keys(value)

    def tap_next(self):
        self.marionette.find_element(*self._next_locator).tap()
        self.wait_for_element_displayed(*self._check_for_new_messages_locator)

    def tap_account_prefs_next(self):
        self.wait_for_element_displayed(*self._account_prefs_next_locator)
        self.marionette.find_element(*self._account_prefs_next_locator).tap()

    def wait_for_setup_complete(self):
        self.wait_for_element_displayed(*self._continue_button_locator)

    def tap_continue(self):
        self.marionette.find_element(*self._continue_button_locator).tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Messages(Base):

    name = 'Messages'

    _create_new_message_locator = (By.ID, 'icon-add')
    _first_message_locator = (By.ID, 'thread-1')

    def launch(self):
        Base.launch(self)
        self.wait_for_element_displayed(*self._create_new_message_locator)

    def tap_create_new_message(self):
        self.marionette.find_element(*self._create_new_message_locator).tap()
        from gaiatest.apps.messages.regions.new_message import NewMessage
        return NewMessage(self.marionette)

    def wait_for_message_list(self):
        self.wait_for_element_displayed(*self._create_new_message_locator)

    def wait_for_message_received(self, timeout=180):
        self.wait_for_element_displayed(*self._first_message_locator, timeout=timeout)

    def tap_first_received_message(self):
        self.marionette.find_element(*self._first_message_locator).tap()
        from gaiatest.apps.messages.regions.message_thread import MessageThread
        return MessageThread(self.marionette)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import smtplib

from email.mime.text import MIMEText


class EmailUtil:

    def send(self, senders_account, mock_email):
        """
        Usage:
        senders_account = dict
        Sender's account details: email, password, smtp_hostname, smtp_port

        mock_email = gaitest.mocks.mock_email.MockObject
        Mock email object.
        """

        self.senders_account = senders_account

        # connect to SMTP server
        self.server = smtplib.SMTP_SSL(self.senders_account['smtp_hostname'],
                                       str(self.senders_account['smtp_port']))
        self.server.set_debuglevel(False)

        self.server.login(self.senders_account['email'],
                          self.senders_account['password'])

        msg = MIMEText(mock_email.message)
        msg['Subject'] = mock_email.subject
        msg['To'] = mock_email.recipients_email
        msg['from'] = self.senders_account['email']

        self.server.sendmail(self.senders_account['email'],
                             mock_email.recipients_email,
                             msg.as_string())

        self.server.quit()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Phone(Base):

    name = "Phone"

    _dialog_locator = (By.ID, 'confirmation-message')
    _dialog_title_locator = (By.XPATH, "//*[@id='confirmation-message']/section/h1")
    _call_log_toolbar_button_locator = (By.ID, 'option-recents')
    _contacts_view_locator = (By.ID, 'option-contacts')
    _keypad_toolbar_button_locator = (By.ID, 'option-keypad')

    _contacts_frame_locator = (By.ID, 'iframe-contacts')

    @property
    def keypad(self):
        from gaiatest.apps.phone.regions.keypad import Keypad
        return Keypad(self.marionette)

    def tap_contacts(self):
        self.marionette.find_element(*self._contacts_view_locator).tap()

        self.wait_for_element_present(*self._contacts_frame_locator)
        frame = self.marionette.find_element(*self._contacts_frame_locator)
        self.marionette.switch_to_frame(frame)

        from gaiatest.apps.contacts.app import Contacts
        return Contacts(self.marionette)

    @property
    def call_screen(self):
        from gaiatest.apps.phone.regions.call_screen import CallScreen
        return CallScreen(self.marionette)

    @property
    def call_log(self):
        from gaiatest.apps.phone.regions.call_log import CallLog
        return CallLog(self.marionette)

    @property
    def confirmation_dialog_text(self):
        return self.marionette.find_element(*self._dialog_title_locator).text

    def wait_for_confirmation_dialog(self):
        self.wait_for_element_displayed(*self._dialog_locator)

    def tap_call_log_toolbar_button(self):
        self.wait_for_element_displayed(*self._call_log_toolbar_button_locator)
        self.marionette.find_element(*self._call_log_toolbar_button_locator).tap()
        return self.call_log

    def tap_keypad_toolbar_button(self):
        self.wait_for_element_displayed(*self._keypad_toolbar_button_locator)
        self.marionette.find_element(*self._keypad_toolbar_button_locator).tap()
        return self.keypad

    def make_call_and_hang_up(self, phone_number):
        """Just makes a call and hangs up. Does not do any assertions."""
        call_screen = self.keypad.call_number(phone_number)
        call_screen.wait_for_outgoing_call()
        call_screen.hang_up()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time

from gaiatest.apps.base import PageRegion


class HTML5Player(PageRegion):
    """Represents HTML5 Player.

    Reference:
    http://www.w3.org/TR/2012/WD-html5-20121025/media-elements.html#media-element
    """

    def wait_for_video_loaded(self):
        self.wait_for_condition(
            lambda m:
            int(self.root_element.get_attribute('readyState')) == 4)

    @property
    def is_paused(self):
        return self.root_element.get_attribute('paused') == 'true'

    @property
    def is_ended(self):
        return self.root_element.get_attribute('ended') == 'true'

    @property
    def has_controls(self):
        return self.root_element.get_attribute('controls') == 'true'

    def _disable_controls(self):
        if self.has_controls:
            self.marionette.execute_script(
                'arguments[0].removeAttribute("controls")',
                script_args=[self.root_element])

    def invoke_controls(self):
        if not self.has_controls:
            self.marionette.execute_script(
                'arguments[0].setAttribute("controls", "controls")',
                script_args=[self.root_element])
            if not (self.is_paused or self.is_ended):
                self.root_element.tap()
        else:
            self.root_element.tap()
        time.sleep(.25)

    def play(self):
        self.invoke_controls()
        self.root_element.tap()
        self.wait_for_condition(lambda m: not self.is_paused)
        self._disable_controls()

    def pause(self):
        self.invoke_controls()
        self.root_element.tap()
        self.wait_for_condition(lambda m: self.is_paused)
        self._disable_controls()

    def is_video_playing(self):
        # get 4 timestamps during approx. 1 sec
        # ensure that newer timestamp has greater value than previous one
        timestamps = []
        for i in range(4):
            timestamps.append(self.current_timestamp)
            time.sleep(.25)
        return all([timestamps[i - 1] < timestamps[i] for i in range(1, 3)])

    @property
    def current_timestamp(self):
        return float(self.root_element.get_attribute('currentTime'))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Persona(Base):

    # iframes
    _persona_frame_locator = (By.CSS_SELECTOR, "iframe.screen[data-url*='persona.org/sign_in#NATIVE']")

    # persona login
    _body_loading_locator = (By.CSS_SELECTOR, 'body.loading')
    _email_input_locator = (By.ID, 'authentication_email')
    _password_input_locator = (By.ID, 'authentication_password')
    _continue_button_locator = (By.CSS_SELECTOR, '.continue.right')
    _returning_button_locator = (By.CSS_SELECTOR, 'button.isReturning')
    _sign_in_button_locator = (By.ID, 'signInButton')
    _this_session_only_button_locator = (By.ID, 'this_is_not_my_computer')
    _this_is_not_me_locator = (By.CSS_SELECTOR, 'p.isMobile a.thisIsNotMe')

    _create_password_locator = (By.ID, 'password')
    _confirm_password_locator = (By.ID, 'vpassword')
    _verify_user_locator = (By.ID, 'verify_user')

    _form_section_locator = (By.CSS_SELECTOR, 'div.vertical div.form_section')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.marionette.switch_to_frame()
        self.wait_for_element_present(*self._persona_frame_locator)

    def login(self, email, password):
        self.switch_to_persona_frame()

        # This is a hack until we are able to run test with a clean profile
        # if a user was logged in tap this is not me
        if self.form_section_id == "selectEmail":
            self.wait_for_sign_in_button()
            self.tap_this_is_not_me()

        self.wait_for_email_input()
        self.type_email(email)
        self.wait_for_continue_button()
        self.tap_continue()

        # if we login with an unverified user we have to confirm the password
        if self.form_section_id == "authentication_form":
            self.wait_for_password_input()
            self.type_password(password)
            self.tap_returning()
        elif self.form_section_id == "set_password":
            self.type_create_password(password)
            self.type_confirm_password(password)
            self.tap_verify_user()
        else:
            raise Exception('Could not log into Persona')

    def switch_to_persona_frame(self):
        self.wait_for_element_present(*self._persona_frame_locator)
        self.frame = self.marionette.find_element(*self._persona_frame_locator)
        self.marionette.switch_to_frame(self.frame)

        self.wait_for_element_not_present(*self._body_loading_locator)

    def type_email(self, value):
        email_field = self.marionette.find_element(*self._email_input_locator)
        email_field.send_keys(value)

    def type_password(self, value):
        password_field = self.marionette.find_element(*self._password_input_locator)
        password_field.send_keys(value)

    def type_create_password(self, value):
        password_field = self.marionette.find_element(*self._create_password_locator)
        password_field.send_keys(value)

    def type_confirm_password(self, value):
        password_field = self.marionette.find_element(*self._confirm_password_locator)
        password_field.send_keys(value)

    def tap_continue(self):
        self.marionette.find_element(*self._continue_button_locator).tap()
        self.wait_for_element_not_displayed(*self._continue_button_locator)

    def tap_verify_user(self):
        self.marionette.find_element(*self._verify_user_locator).tap()

    def tap_sign_in(self):
        self.marionette.find_element(*self._sign_in_button_locator).tap()

    def tap_this_is_not_me(self):
        self.marionette.find_element(*self._this_is_not_me_locator).tap()

    def tap_returning(self):
        self.marionette.find_element(*self._returning_button_locator).tap()

    def tap_this_session_only(self):
        self.marionette.find_element(*self._this_session_only_button_locator).tap()

    @property
    def form_section_id(self):
        self.wait_for_element_displayed(*self._form_section_locator)
        return self.marionette.find_element(*self._form_section_locator).get_attribute('id')

    def wait_for_sign_in_button(self):
        self.wait_for_element_displayed(*self._sign_in_button_locator)

    def wait_for_email_input(self):
        self.wait_for_element_displayed(*self._email_input_locator)

    def wait_for_password_input(self):
        self.wait_for_element_displayed(*self._password_input_locator)

    def wait_for_continue_button(self):
        self.wait_for_element_displayed(*self._continue_button_locator)


# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import sys
import textwrap
import time

from yoctopuce.yocto_api import YAPI, YRefParam
from yoctopuce.yocto_current import YCurrent
from yoctopuce.yocto_datalogger import YDataLogger

class GaiaOptionsMixin(object):

    def gaia_verify_usage(self, options, tests):
        if options.yocto:
            # need to verify that the yocto ammeter is attached
            errmsg = YRefParam()
            if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
                raise RuntimeError('could not register yocto usb connection: %s' % str(errmsg))

            # rescan for yocto devices
            if YAPI.UpdateDeviceList(errmsg) != YAPI.SUCCESS:
                raise RuntimeError('could not detect yoctopuce modules: %s' % str(errmsg))

            # check for ammeter
            ammeter = YCurrent.FirstCurrent()
            if ammeter is None:
                raise RuntimeError('could not find ammeter device')
            if ammeter.isOnline():
                module = ammeter.get_module()
                dc_ammeter = YCurrent.FindCurrent(module.get_serialNumber() + '.current1')
                if (not module.isOnline()) or (dc_ammeter is None):
                    raise RuntimeError('could not get ammeter device')
            else:
                raise RuntimeError('could not find yocto ammeter device')

            # check for data logger
            data_logger = YDataLogger.FirstDataLogger()
            if data_logger is None :
                raise RuntimeError('could not find data logger device')
            if data_logger.isOnline():
                module = data_logger.get_module()
                data_logger = YDataLogger.FindDataLogger(module.get_serialNumber() + '.dataLogger')
                if not module.isOnline() or data_logger is None:
                    raise RuntimeError('could not get data logger device')
            else:
                raise RuntimeError('could not find yocto ammeter device')

    def __init__(self, **kwargs):
        # Inheriting object must call this __init__ to set up option handling
        group = self.add_option_group('gaiatest')
        group.add_option('--restart',
                         action='store_true',
                         dest='restart',
                         default=False,
                         help='restart target instance between tests')
        group.add_option('--yocto',
                         action='store_true',
                         dest='yocto',
                         default=False,
                         help='collect voltage and amperage during test runs (requires special hardware)')
        self.verify_usage_handlers.append(self.gaia_verify_usage)


class GaiaTestRunnerMixin(object):

    def __init__(self, **kwargs):
        width = 80
        if not (self.testvars.get('acknowledged_risks') is True or os.environ.get('GAIATEST_ACKNOWLEDGED_RISKS')):
            url = 'https://developer.mozilla.org/en-US/docs/Gaia_Test_Runner#Risks'
            heading = 'Acknowledge risks'
            message = 'These tests are destructive and may remove data from the target Firefox OS instance as well ' \
                      'as using services that may incur costs! Before you can run these tests you must follow the ' \
                      'steps to indicate you have acknowledged the risks detailed at the following address:'
            print '\n' + '*' * 5 + ' %s ' % heading.upper() + '*' * (width - len(heading) - 7)
            print '\n'.join(textwrap.wrap(message, width))
            print url
            print '*' * width + '\n'
            sys.exit(1)
        if not (self.testvars.get('skip_warning') is True or os.environ.get('GAIATEST_SKIP_WARNING')):
            delay = 30
            heading = 'Warning'
            message = 'You are about to run destructive tests against a Firefox OS instance. These tests ' \
                      'will restore the target to a clean state, meaning any personal data such as contacts, ' \
                      'messages, photos, videos, music, etc. will be removed. This may include data on the ' \
                      'microSD card. The tests may also attempt to initiate outgoing calls, or connect to ' \
                      'services such as cellular data, wifi, gps, bluetooth, etc.'
            try:
                print '\n' + '*' * 5 + ' %s ' % heading.upper() + '*' * (width - len(heading) - 7)
                print '\n'.join(textwrap.wrap(message, width))
                print '*' * width + '\n'
                print 'To abort the test run hit Ctrl+C on your keyboard.'
                print 'The test run will continue in %d seconds.' % delay
                time.sleep(delay)
            except KeyboardInterrupt:
                print '\nTest run aborted by user.'
                sys.exit(1)
            print 'Continuing with test run...\n'




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from gaiatest.apps.base import Base

class TestContainer(Base):

    name = 'Test Container'

    def launch(self):
        Base.launch(self)




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time
from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion
import gaiatest.apps.gallery.app


class Camera(Base):

    name = 'Camera'

    _capture_button_locator = (By.CSS_SELECTOR, '#capture-button:not([disabled])')
    _filmstrip_thumbnail_locator = (By.CSS_SELECTOR, '#filmstrip > img.thumbnail')
    _switch_source_button_locator = (By.ID, 'switch-button')
    _video_timer_locator = (By.ID, 'video-timer')
    _filmstrip_locator = (By.ID, 'filmstrip')
    _body_locator = (By.TAG_NAME, 'body')
    _gallery_button_locator = (By.ID, 'gallery-button')

    # locators for the case when other apps open camera app
    _select_button_locator = (By.ID, 'select-button')
    _camera_frame_locator = (By.CSS_SELECTOR, 'iframe[src*="camera"][src*="/index.html"]')

    def launch(self):
        Base.launch(self)
        self.wait_for_camera_ready()

    def take_photo(self):
        self.tap_capture()

        # Wait for filmstrip to appear
        self.wait_for_filmstrip_visible()

        # Wait for filmstrip to clear
        self.wait_for_filmstrip_not_visible()

    def record_video(self, duration):
        # Start recording
        self.tap_capture()
        self.wait_for_video_capturing()
        # Wait for duration
        timer_text = "00:%02d" % duration
        self.wait_for_condition(lambda m: m.find_element(
            *self._video_timer_locator).text == timer_text, timeout=duration + 30)
        # Stop recording
        self.tap_capture()
        self.wait_for_element_not_displayed(*self._video_timer_locator)

    def tap_capture(self):
        self.wait_for_camera_ready()
        self.marionette.find_element(*self._capture_button_locator).tap()

    def tap_select_button(self):
        self.marionette.find_element(*self._select_button_locator).tap()

    def tap_switch_source(self):
        self.marionette.find_element(*self._switch_source_button_locator).tap()

    def tap_to_display_filmstrip(self):
        self.marionette.find_element(*self._body_locator).tap()
        self.wait_for_filmstrip_visible()

    def wait_for_select_button_displayed(self):
        self.wait_for_element_displayed(*self._select_button_locator)

    def wait_for_camera_ready(self):
        self.wait_for_element_displayed(*self._capture_button_locator)

    def wait_for_filmstrip_visible(self):
        self.wait_for_condition(lambda m: self.is_filmstrip_visible)

    def wait_for_filmstrip_not_visible(self):
        filmstrip = self.marionette.find_element(*self._filmstrip_locator)
        self.wait_for_condition(lambda m: filmstrip.location['y'] == (0 - filmstrip.size['height']), timeout=10)

    def wait_for_video_capturing(self):
        body = self.marionette.find_element(*self._body_locator)
        self.wait_for_condition(lambda m: 'capturing' in body.get_attribute('class'))

    def wait_for_video_timer_not_visible(self):
        self.wait_for_element_not_displayed(*self._video_timer_locator)

    def switch_to_camera_frame(self):
        self.marionette.switch_to_frame()
        self.wait_for_element_present(*self._camera_frame_locator)
        camera_frame = self.marionette.find_element(*self._camera_frame_locator)
        self.marionette.switch_to_frame(camera_frame)
        self.wait_for_camera_ready()

    def tap_switch_to_gallery(self):
        switch_to_gallery_button = self.marionette.find_element(*self._gallery_button_locator)
        switch_to_gallery_button.tap()
        gallery_app = gaiatest.apps.gallery.app.Gallery(self.marionette)
        self.wait_for_condition(lambda m: self.apps.displayed_app.name == gallery_app.name)
        self.apps.switch_to_displayed_app()
        return gallery_app

    @property
    def is_filmstrip_visible(self):
        return self.marionette.find_element(*self._filmstrip_locator).location['y'] == 0

    @property
    def video_timer(self):
        text = self.marionette.find_element(*self._video_timer_locator).text
        return time.strptime(text, '%M:%S')

    @property
    def is_gallery_button_visible(self):
        return self.is_element_displayed(*self._gallery_button_locator)

    @property
    def filmstrip_images(self):
        return [FilmStripImage(self.marionette, image)
                for image in self.marionette.find_elements(*self._filmstrip_thumbnail_locator)]


class FilmStripImage(PageRegion):

    def tap(self):
        image_preview = ImagePreview(self.marionette)
        self.root_element.tap()
        image_preview.wait_for_media_frame()
        return image_preview


class ImagePreview(Base):

    _media_frame_locator = (By.ID, 'preview')
    _image_preview_locator = (By.CSS_SELECTOR, '#media-frame > img')
    _camera_button_locator = (By.ID, 'camera-button')

    @property
    def is_image_preview_visible(self):
        return self.is_element_displayed(*self._image_preview_locator)

    def wait_for_media_frame(self):
        media_frame = self.marionette.find_element(*self._media_frame_locator)
        scr_height = int(self.marionette.execute_script('return window.screen.height'))
        self.wait_for_condition(lambda m: (media_frame.location['y'] + media_frame.size['height']) == scr_height)

    def tap_camera(self):
        self.marionette.find_element(*self._camera_button_locator).tap()
        camera = Camera(self.marionette)
        return camera



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from marionette.marionette import Actions
from gaiatest.apps.base import Base


class FullscreenImage(Base):

    _fullscreen_view_locator = (By.ID, 'fullscreen-view')
    _current_image_locator = (By.CSS_SELECTOR, '#frames > div.frame[style ~= "translateX(0px);"] > img')
    _photos_toolbar_locator = (By.ID, 'fullscreen-toolbar')
    _delete_image_locator = (By.ID, 'fullscreen-delete-button')
    _confirm_delete_locator = (By.ID, 'modal-dialog-confirm-ok')
    _edit_photo_locator = (By.ID, 'fullscreen-edit-button')
    _tile_view_locator = (By.ID, 'fullscreen-back-button')
    _share_button_locator = (By.ID, 'fullscreen-share-button')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._current_image_locator)

    @property
    def is_photo_toolbar_displayed(self):
        return self.marionette.find_element(*self._photos_toolbar_locator).is_displayed()

    @property
    def current_image_source(self):
        return self.marionette.find_element(*self._current_image_locator).get_attribute('src')

    def flick_to_next_image(self):
        self._flick_to_image('next')

    def flick_to_previous_image(self):
        self._flick_to_image('previous')

    def _flick_to_image(self, direction):
        action = Actions(self.marionette)

        current_image = self.marionette.find_element(*self._current_image_locator)
        current_image_move_x = current_image.size['width'] / 2
        current_image_mid_x = current_image.size['width'] / 2
        current_image_mid_y = current_image.size['height'] / 2

        if direction == 'next':
            action.flick(current_image, current_image_mid_x, current_image_mid_y, current_image_mid_x - current_image_move_x, current_image_mid_y)
        else:
            action.flick(current_image, current_image_mid_x, current_image_mid_y, current_image_mid_x + current_image_move_x, current_image_mid_y)

        action.perform()
        self.wait_for_element_displayed(*self._current_image_locator)

    def tap_delete_button(self):
        self.marionette.find_element(*self._delete_image_locator).tap()
        self.marionette.switch_to_frame()
        self.wait_for_element_displayed(*self._confirm_delete_locator)

    def tap_confirm_deletion_button(self):
        self.marionette.find_element(*self._confirm_delete_locator).tap()
        self.wait_for_element_not_displayed(*self._confirm_delete_locator)
        from gaiatest.apps.gallery.app import Gallery
        gallery = Gallery(self.marionette)
        self.wait_for_condition(lambda m: self.apps.displayed_app.name == gallery.name)
        self.marionette.switch_to_frame(self.apps.displayed_app.frame)
        return gallery

    def tap_edit_button(self):
        self.marionette.find_element(*self._edit_photo_locator).tap()
        from gaiatest.apps.gallery.regions.edit_photo import EditPhoto
        return EditPhoto(self.marionette)

    def tap_tile_view_button(self):
        self.marionette.find_element(*self._tile_view_locator).tap()
        self.wait_for_element_not_displayed(*self._fullscreen_view_locator)
        from gaiatest.apps.gallery.app import Gallery
        return Gallery(self.marionette)

    def tap_share_button(self):
        self.marionette.find_element(*self._share_button_locator).tap()
        from gaiatest.apps.system.regions.activities import Activities
        return Activities(self.marionette)

    @property
    def photo_toolbar_width(self):
        return self.marionette.execute_script('return document.getElementById("fullscreen-toolbar").offsetWidth')

    @property
    def is_image_displayed(self):
        return self.is_element_displayed(*self._current_image_locator)

    @property
    def current_scale(self):
        style = self.marionette.find_element(*self._current_image_locator).get_attribute('style')
        return style.split('scale(')[1].split(') ')[0]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class EditPhoto(Base):

    _edit_view_locator = (By.ID, 'edit-view')
    _edit_effect_button_locator = (By.ID, 'edit-effect-button')
    _effect_options_locator = (By.CSS_SELECTOR, '#edit-effect-options a')
    _edit_save_locator = (By.ID, 'edit-save-button')
    _edit_crop_button_locator = (By.ID, 'edit-crop-button')
    _crop_portrait_locator = (By.ID, 'edit-crop-aspect-portrait')
    _save_progress_bar_locator = (By.ID, 'save-progress')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._edit_view_locator)

    def tap_edit_effects_button(self):
        self.marionette.find_element(*self._edit_effect_button_locator).tap()
        self.wait_for_element_displayed(*self._effect_options_locator)

    def tap_edit_crop_button(self):
        self.marionette.find_element(*self._edit_crop_button_locator).tap()
        self.wait_for_element_displayed(*self._crop_portrait_locator)

    def tap_edit_save_button(self):
        self.marionette.find_element(*self._edit_save_locator).tap()
        self.wait_for_element_not_displayed(*self._save_progress_bar_locator)
        from gaiatest.apps.gallery.app import Gallery
        return Gallery(self.marionette)

    def tap_portrait_crop(self):
        self.marionette.find_element(*self._crop_portrait_locator).tap()
        self.wait_for_condition(lambda m: 'selected' in self.marionette.find_element(*self._crop_portrait_locator).get_attribute('class'))

    @property
    def effects(self):
        return [self.Effect(marionette=self.marionette, element=effect)
                for effect in self.marionette.find_elements(*self._effect_options_locator)]

    class Effect(PageRegion):

        def tap(self):
            self.root_element.tap()
            self.wait_for_condition(lambda m: 'selected' in self.root_element.get_attribute('class'))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class CellDataPrompt(Base):

    _cell_data_prompt_container_locator = (By.CSS_SELECTOR, '#carrier-dc-warning')
    _cell_data_prompt_turn_on_button_locator = (By.CSS_SELECTOR, '#carrier-dc-warning button[type="submit"]')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._cell_data_prompt_turn_on_button_locator)

    def turn_on(self):
        self.marionette.find_element(*self._cell_data_prompt_turn_on_button_locator).tap()

    @property
    def is_displayed(self):
        # The prompt container and its contents return True is_displayed erroneously
        return 'current' in self.marionette.find_element(*self._cell_data_prompt_container_locator).get_attribute('class')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Settings(Base):

    name = 'Settings'

    _header_text_locator = (By.CSS_SELECTOR, '#root > header > h1')
    _data_text_locator = (By.ID, 'data-desc')
    _airplane_switch_locator = (By.XPATH, "//input[@id='airplaneMode-input']/..")
    _airplane_checkbox_locator = (By.ID, "airplaneMode-input")
    _wifi_text_locator = (By.ID, 'wifi-desc')
    _gps_enabled_locator = (By.XPATH, "//input[@name='geolocation.enabled']")
    _gps_switch_locator = (By.XPATH, "//input[@name='geolocation.enabled']/..")
    _cell_data_menu_item_locator = (By.ID, 'menuItem-cellularAndData')
    _bluetooth_menu_item_locator = (By.ID, 'menuItem-bluetooth')
    _keyboard_menu_item_locator = (By.ID, "menuItem-keyboard")
    _language_menu_item_locator = (By.ID, 'menuItem-languageAndRegion')
    _do_not_track_menu_item_locator = (By.ID, 'menuItem-doNotTrack')
    _media_storage_menu_item_locator = (By.ID, 'menuItem-mediaStorage')
    _phone_lock_menu_item_locator = (By.ID, 'menuItem-phoneLock')
    _display_menu_item_locator = (By.ID, 'menuItem-display')
    _wifi_menu_item_locator = (By.ID, 'menuItem-wifi')
    _device_info_menu_item_locator = (By.ID, 'menuItem-deviceInfo')
    _app_permissions_menu_item_locator = (By.ID, 'menuItem-appPermissions')
    _battery_menu_item_locator = (By.ID, 'menuItem-battery')

    def launch(self):
        Base.launch(self)
        checkbox = self.marionette.find_element(*self._airplane_checkbox_locator)
        self.wait_for_condition(lambda m: checkbox.is_enabled())

    def toggle_airplane_mode(self):
        checkbox = self.marionette.find_element(*self._airplane_checkbox_locator)
        label = self.marionette.find_element(*self._airplane_switch_locator)

        checkbox_state = checkbox.is_selected()

        label.tap()
        self.wait_for_condition(lambda m: checkbox_state is not checkbox.is_selected())

    def enable_gps(self):
        self.marionette.find_element(*self._gps_switch_locator).tap()
        self.wait_for_condition(lambda m: self.is_gps_enabled)

    def disable_gps(self):
        self.marionette.find_element(*self._gps_switch_locator).tap()
        self.wait_for_condition(lambda m: not self.is_gps_enabled)

    @property
    def is_gps_enabled(self):
        checkbox = self.marionette.find_element(*self._gps_enabled_locator)
        return checkbox.is_selected()

    @property
    def header_text(self):
        return self.marionette.find_element(*self._header_text_locator).text

    @property
    def cell_data_menu_item_description(self):
        return self.marionette.find_element(*self._data_text_locator).text

    @property
    def wifi_menu_item_description(self):
        return self.marionette.find_element(*self._wifi_text_locator).text

    def open_cell_and_data_settings(self):
        from gaiatest.apps.settings.regions.cell_data import CellData
        self._tap_menu_item(self._cell_data_menu_item_locator)
        return CellData(self.marionette)

    def open_bluetooth_settings(self):
        from gaiatest.apps.settings.regions.bluetooth import Bluetooth
        # this is technically visible, but needs scroll to be tapped
        # TODO Remove when bug 937053 is resolved
        bluetooth_menu_item = self.marionette.find_element(*self._bluetooth_menu_item_locator)
        self.marionette.execute_script("arguments[0].scrollIntoView(false);", [bluetooth_menu_item])
        self._tap_menu_item(self._bluetooth_menu_item_locator)
        return Bluetooth(self.marionette)

    def open_keyboard_settings(self):
        from gaiatest.apps.settings.regions.keyboard import Keyboard
        self._tap_menu_item(self._keyboard_menu_item_locator)
        return Keyboard(self.marionette)

    def open_language_settings(self):
        from gaiatest.apps.settings.regions.language import Language
        self._tap_menu_item(self._language_menu_item_locator)
        return Language(self.marionette)

    def open_do_not_track_settings(self):
        from gaiatest.apps.settings.regions.do_not_track import DoNotTrack
        self._tap_menu_item(self._do_not_track_menu_item_locator)
        return DoNotTrack(self.marionette)

    def open_media_storage_settings(self):
        from gaiatest.apps.settings.regions.media_storage import MediaStorage
        self._tap_menu_item(self._media_storage_menu_item_locator)
        return MediaStorage(self.marionette)

    def open_phone_lock_settings(self):
        from gaiatest.apps.settings.regions.phone_lock import PhoneLock
        self._tap_menu_item(self._phone_lock_menu_item_locator)
        return PhoneLock(self.marionette)

    def open_display_settings(self):
        from gaiatest.apps.settings.regions.display import Display
        self._tap_menu_item(self._display_menu_item_locator)
        return Display(self.marionette)

    def open_wifi_settings(self):
        from gaiatest.apps.settings.regions.wifi import Wifi
        self._tap_menu_item(self._wifi_menu_item_locator)
        return Wifi(self.marionette)

    def open_device_info_settings(self):
        from gaiatest.apps.settings.regions.device_info import DeviceInfo
        self._tap_menu_item(self._device_info_menu_item_locator)
        return DeviceInfo(self.marionette)

    def open_app_permissions_settings(self):
        from gaiatest.apps.settings.regions.app_permissions import AppPermissions
        self._tap_menu_item(self._app_permissions_menu_item_locator)
        return AppPermissions(self.marionette)

    def open_battery_settings(self):
        from gaiatest.apps.settings.regions.battery import Battery
        self._tap_menu_item(self._battery_menu_item_locator)
        return Battery(self.marionette)

    def _tap_menu_item(self, menu_item_locator):
        self.wait_for_element_displayed(*menu_item_locator)
        menu_item = self.marionette.find_element(*menu_item_locator)
        parent_section = menu_item.find_element(By.XPATH, 'ancestor::section')
        menu_item.tap()
        self.wait_for_condition(lambda m: parent_section.location['x'] + parent_section.size['width'] == 0)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time

from marionette.by import By
from marionette.marionette import Actions

from gaiatest.apps.clock.app import Clock


class NewAlarm(Clock):

    _alarm_name_locator = (By.ID, 'alarm-name')
    _repeat_menu_locator = (By.ID, 'repeat-menu')
    _sound_menu_locator = (By.ID, 'sound-menu')
    _snooze_menu_locator = (By.ID, 'snooze-menu')
    _done_locator = (By.ID, 'alarm-done')
    _time_button_locator = (By.XPATH, "//li[input[@id='time-select']]")

    _hour_picker_locator = (By.CSS_SELECTOR, '#value-picker-hours div')
    _minutes_picker_locator = (By.CSS_SELECTOR, '#value-picker-minutes div')
    _hour24_picker_locator = (By.CSS_SELECTOR, '#value-picker-hour24-state div')

    def type_alarm_label(self, value):
        label = self.marionette.find_element(*self._alarm_name_locator)
        label.clear()
        label.send_keys(value)
        self.keyboard.dismiss()

    @property
    def alarm_label_placeholder(self):
        return self.marionette.find_element(*self._alarm_name_locator).get_attribute('placeholder')

    @property
    def alarm_repeat(self):
        return self.marionette.find_element(*self._repeat_menu_locator).text

    def select_repeat(self, value):
        self.marionette.find_element(*self._repeat_menu_locator).tap()
        self.select(value)

    @property
    def alarm_snooze(self):
        return self.marionette.find_element(*self._snooze_menu_locator).text

    def select_snooze(self, value):
        self.marionette.find_element(*self._snooze_menu_locator).tap()
        self.select(value)

    @property
    def alarm_sound(self):
        return self.marionette.find_element(*self._sound_menu_locator).text

    def select_sound(self, value):
        self.marionette.find_element(*self._sound_menu_locator).tap()
        self.select(value)

    def wait_for_panel_to_load(self):
        screen_width = int(self.marionette.execute_script('return window.innerWidth'))
        done = self.marionette.find_element(*self._done_locator)
        self.wait_for_condition(lambda m: (done.location['x'] + done.size['width']) == screen_width)

    def tap_done(self):
        self.marionette.find_element(*self._done_locator).tap()

        clock = Clock(self.marionette)
        clock.wait_for_banner_displayed()
        return clock

    @property
    def hour(self):
        return self.marionette.find_element(*self._current_element(*self._hour_picker_locator)).text

    def spin_hour(self):
        self.wait_for_element_displayed(*self._hour_picker_locator)
        if int(self.hour) > 6:
                self._flick_menu_down(self._hour_picker_locator)
        else:
            self._flick_menu_up(self._hour_picker_locator)
        time.sleep(1)

    @property
    def minute(self):
        return self.marionette.find_element(*self._current_element(*self._minutes_picker_locator)).text

    def spin_minute(self):
        if int(self.minute) > 30:
            self._flick_menu_down(self._minutes_picker_locator)
        else:
            self._flick_menu_up(self._minutes_picker_locator)

        time.sleep(1)

    @property
    def hour24(self):
        return self.marionette.find_element(*self._current_element(*self._hour24_picker_locator)).text

    def spin_hour24(self):
        hour24_picker = self.marionette.find_element(*self._current_element(*self._hour24_picker_locator))
        hour24_picker_move_y = hour24_picker.size['height'] * 2
        hour24_picker_mid_x = hour24_picker.size['width'] / 2
        hour24_picker_mid_y = hour24_picker.size['height'] / 2

        if self.hour24 == 'AM':
            Actions(self.marionette).flick(hour24_picker, hour24_picker_mid_x, hour24_picker_mid_y, hour24_picker_mid_x, hour24_picker_mid_y - hour24_picker_move_y)
        else:
            Actions(self.marionette).flick(hour24_picker, hour24_picker_mid_x, hour24_picker_mid_y, hour24_picker_mid_x, hour24_picker_mid_y + hour24_picker_move_y)

        time.sleep(1)

    def tap_time(self):
        self.marionette.find_element(*self._time_button_locator).tap()

    def _flick_menu_up(self, locator):
        self.wait_for_element_displayed(*self._current_element(*locator))
        current_element = self.marionette.find_element(*self._current_element(*locator))
        next_element = self.marionette.find_element(*self._next_element(*locator))

        #TODO: update this with more accurate Actions
        action = Actions(self.marionette)
        action.press(next_element)
        action.move(current_element)
        action.release()
        action.perform()

    def _flick_menu_down(self, locator):
        self.wait_for_element_displayed(*self._current_element(*locator))
        current_element = self.marionette.find_element(*self._current_element(*locator))
        next_element = self.marionette.find_element(*self._next_element(*locator))

        #TODO: update this with more accurate Actions
        action = Actions(self.marionette)
        action.press(current_element)
        action.move(next_element)
        action.release()
        action.perform()

    def _current_element(self, method, target):
        return (method, '%s.picker-unit.active' % target)

    def _next_element(self, method, target):
        return (method, '%s.picker-unit.active + div' % target)


class EditAlarm(NewAlarm):

    _alarm_delete_button_locator = (By.ID, 'alarm-delete')

    def __init__(self, marionette):
        NewAlarm.__init__(self, marionette)
        self.wait_for_element_displayed(*self._alarm_delete_button_locator)

    def tap_delete(self):
        self.marionette.find_element(*self._alarm_delete_button_locator).tap()
        return Clock(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time

from marionette.by import By
from marionette.marionette import Actions

from gaiatest.apps.base import Base
from gaiatest.apps.phone.app import Phone
from gaiatest.apps.phone.regions.call_screen import CallScreen


class Keypad(Phone):

    #locators
    _keyboard_container_locator = (By.ID, 'keyboard-container')
    _phone_number_view_locator = (By.ID, 'phone-number-view')
    _call_bar_locator = (By.ID, 'keypad-callbar-call-action')
    _add_new_contact_button_locator = (By.ID, 'keypad-callbar-add-contact')

    def __init__(self, marionette):
        Phone.__init__(self, marionette)

    @property
    def phone_number(self):
        return self.marionette.find_element(*self._phone_number_view_locator).get_attribute('value')

    def dial_phone_number(self, value):
        for i in value:
            if i == "+":
                zero_button = self.marionette.find_element(By.CSS_SELECTOR, 'div.keypad-key[data-value="0"]')
                Actions(self.marionette).long_press(zero_button, 1.2).perform()
            else:
                self.marionette.find_element(By.CSS_SELECTOR, 'div.keypad-key[data-value="%s"]' % i).tap()
                time.sleep(0.25)

    def call_number(self, value):
        self.dial_phone_number(value)
        return self.tap_call_button()

    def tap_call_button(self, switch_to_call_screen=True):
        self.marionette.find_element(*self._call_bar_locator).tap()
        if switch_to_call_screen:
            return CallScreen(self.marionette)

    def tap_add_contact(self):
        self.marionette.find_element(*self._add_new_contact_button_locator).tap()
        return AddNewNumber(self.marionette)

    def switch_to_keypad_frame(self):
        app = self.apps.displayed_app
        self.marionette.switch_to_frame(app.frame)


class AddNewNumber(Base):
    _create_new_contact_locator = (By.ID, 'create-new-contact-menuitem')
    _new_contact_frame_locator = (By.CSS_SELECTOR, "iframe[src^='app://communications'][src$='contacts/index.html?new']")

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._create_new_contact_locator)

    def tap_create_new_contact(self):
        self.marionette.find_element(*self._create_new_contact_locator).tap()

        self.marionette.switch_to_frame()
        self.wait_for_element_present(*self._new_contact_frame_locator)
        frame = self.marionette.find_element(*self._new_contact_frame_locator)
        self.marionette.switch_to_frame(frame)

        from gaiatest.apps.contacts.regions.contact_form import NewContact
        return NewContact(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class SettingsForm(Base):

    _loading_overlay_locator = (By.ID, 'loading-overlay')
    _settings_close_button_locator = (By.ID, 'settings-close')
    _order_by_last_name_locator = (By.CSS_SELECTOR, 'p[data-l10n-id="contactsOrderBy"]')
    _order_by_last_name_switch_locator = (By.CSS_SELECTOR, 'input[name="order.lastname"]')
    _import_from_sim_button_locator = (By.CSS_SELECTOR, "li[id*='import-sim-option'] button")
    _import_from_sdcard_locator = (By.CSS_SELECTOR, 'button.icon-sd[data-l10n-id="importMemoryCard"]')
    _import_from_gmail_button_locator = (By.CSS_SELECTOR, 'button.icon-gmail[data-l10n-id="importGmail"]')
    _import_from_windows_live_button_locator = (By.CSS_SELECTOR, 'button.icon-live[data-l10n-id="importLive"]')
    _back_from_import_contacts_locator = (By.ID, 'import-settings-back')
    _export_to_sd_button_locator = (By.CSS_SELECTOR, 'button[data-l10n-id="memoryCard"]')
    _import_contacts_locator = (By.CSS_SELECTOR, 'button[data-l10n-id="importContactsTitle"]')
    _export_contacts_locator = (By.CSS_SELECTOR, 'button[data-l10n-id="exportContactsTitle"]')
    _gmail_contacts_imported_locator = (By.CSS_SELECTOR, '.icon.icon-gmail > p > span')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._settings_close_button_locator)

    def tap_order_by_last_name(self):
        self.wait_for_element_displayed(*self._order_by_last_name_locator)
        self.marionette.find_element(*self._order_by_last_name_locator).click()

    @property
    def order_by_last_name(self):
        return self.marionette.find_element(*self._order_by_last_name_switch_locator).is_selected()

    def tap_import_contacts(self):
        self.wait_for_element_displayed(*self._import_contacts_locator)
        self.marionette.find_element(*self._import_contacts_locator).tap()

    def tap_export_contacts(self):
        self.wait_for_element_displayed(*self._export_contacts_locator)
        self.marionette.find_element(*self._export_contacts_locator).tap()

    def tap_import_from_sim(self):
        self.wait_for_element_displayed(*self._import_from_sim_button_locator)
        self.marionette.find_element(*self._import_from_sim_button_locator).tap()
        self.wait_for_element_not_displayed(*self._loading_overlay_locator)

    @property
    def gmail_imported_contacts(self):
        return self.marionette.find_element(*self._gmail_contacts_imported_locator).text

    def tap_import_from_gmail(self):
        self.wait_for_element_displayed(*self._import_from_gmail_button_locator)
        self.marionette.find_element(*self._import_from_gmail_button_locator).tap()
        from gaiatest.apps.contacts.regions.gmail import GmailLogin
        return GmailLogin(self.marionette)

    def tap_import_from_sdcard(self):
        self.wait_for_element_displayed(*self._import_from_sdcard_locator)
        self.marionette.find_element(*self._import_from_sdcard_locator).tap()
        self.wait_for_element_not_displayed(*self._loading_overlay_locator)

    def tap_export_to_sd(self):
        self.wait_for_element_displayed(*self._export_to_sd_button_locator)
        self.marionette.find_element(*self._export_to_sd_button_locator).tap()
        self.wait_for_element_not_displayed(*self._loading_overlay_locator)

    def tap_done(self):
        self.marionette.find_element(*self._settings_close_button_locator).tap()
        self.wait_for_element_not_displayed(*self._settings_close_button_locator)
        from gaiatest.apps.contacts.app import Contacts
        return Contacts(self.marionette)

    def tap_back_from_import_contacts(self):
        self.marionette.find_element(*self._back_from_import_contacts_locator).tap()
        self.wait_for_element_not_displayed(*self._back_from_import_contacts_locator)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Language(Base):

    _select_language_locator = (By.CSS_SELECTOR, "select[name='language.current']")
    _back_button_locator = (By.CSS_SELECTOR, '.current header > a')

    def go_back(self):
        self.marionette.find_element(*self._back_button_locator).tap()

    def select_language(self, language):
        self.marionette.find_element(*self._select_language_locator).tap()
        self.select(language)

    def select(self, match_string):
        # This needs to be duplicated from base.py for a few reasons:
        # 1. When we return from the frame we don't return to the Settings app in its initial state,
        #    so the wait for in its launch method times out
        # 2. We need to use in instead of == on the match text because of the directional strings

        # have to go back to top level to get the B2G select box wrapper
        self.marionette.switch_to_frame()

        self.wait_for_condition(lambda m: len(self.marionette.find_elements(By.CSS_SELECTOR, '#value-selector-container li')) > 0)

        options = self.marionette.find_elements(By.CSS_SELECTOR, '#value-selector-container li')
        close_button = self.marionette.find_element(By.CSS_SELECTOR, 'button.value-option-confirm')

        # loop options until we find the match
        for li in options:
            if match_string in li.text:
                # TODO Remove scrollintoView upon resolution of bug 877651
                self.marionette.execute_script(
                    'arguments[0].scrollIntoView(false);', [li])
                li.tap()
                break
        else:
            raise Exception("Element '%s' could not be found in select wrapper" % match_string)

        close_button.tap()

        # now back to app
        self.marionette.switch_to_frame(self.apps.displayed_app.frame)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from marionette.by import By
from gaiatest.apps.base import Base

from marionette.marionette import Actions


class ContextMenuPage(Base):
    _context_menu_body_locator = (By.CSS_SELECTOR, "body > section")
    _frame_locator = (By.CSS_SELECTOR, "#test-iframe[src*='contextmenu']")

    def __init__(self, marionette):
        Base.__init__(self, marionette)

    def switch_to_frame(self):
        self.wait_for_element_displayed(*self._frame_locator)
        context_menu_page_iframe = self.marionette.find_element(*self._frame_locator)
        self.marionette.switch_to_frame(context_menu_page_iframe)

    def long_press_context_menu_body(self):
        self.wait_for_condition(lambda m: m.find_element(*self._context_menu_body_locator).is_displayed())
        context_menu_body = self.marionette.find_element(*self._context_menu_body_locator)
        Actions(self.marionette).press(context_menu_body).wait(1).release().perform()

        from gaiatest.apps.system.regions.activities import Activities

        return Activities(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class GmailLogin(Base):

    _gmail_sign_in_frame_locator = (By.CSS_SELECTOR, '#frame-container > iframe[data-url*="google"]')
    _email_locator = (By.ID, 'Email')
    _password_locator = (By.ID, 'Passwd')
    _sign_in_locator = (By.ID, 'signIn')

    def switch_to_gmail_login_frame(self):
        self.marionette.switch_to_frame()
        gmail_sign_in = self.marionette.find_element(*self._gmail_sign_in_frame_locator)
        self.marionette.switch_to_frame(gmail_sign_in)

    def gmail_login(self, user, passwd):
        self.wait_for_element_displayed(*self._email_locator)
        self.marionette.find_element(*self._email_locator).tap()
        self.marionette.find_element(*self._email_locator).send_keys(user)
        self.marionette.find_element(*self._password_locator).tap()
        self.marionette.find_element(*self._password_locator).send_keys(passwd)
        self.marionette.find_element(*self._sign_in_locator).tap()
        from gaiatest.apps.contacts.app import Contacts
        return Contacts(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class Settings(Base):
    #general email settings
    _email_account_locator = (By.CSS_SELECTOR, '.tng-account-item')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._email_account_locator)

    @property
    def email_accounts(self):
        return [self.Account(self.marionette, email_account) for email_account in self.marionette.find_elements(*self._email_account_locator)]

    class Account(PageRegion):
        _name_locator = (By.CSS_SELECTOR, 'a.tng-account-item-label')

        def tap(self):
            self.root_element.tap()
            return EmailAccountSettings(self.marionette)


class EmailAccountSettings(Base):
    #settings for a specific email account

    _delete_account_locator = (By.CSS_SELECTOR, '.tng-account-delete')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._delete_account_locator)

    def tap_delete(self):
        self.marionette.find_element(*self._delete_account_locator).tap()
        return DeleteConfirmation(self.marionette)


class DeleteConfirmation(Base):
    _delete_locator = (By.CSS_SELECTOR, 'body > .tng-account-delete-confirm #account-delete-ok')
    _cancel_locator = (By.CSS_SELECTOR, 'body > .tng-account-delete-confirm #account-delete-cancel')
    _message_locator = (By.CSS_SELECTOR, 'body > .tng-account-delete-confirm > section > p')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._delete_locator)

    def tap_delete(self):
        self.marionette.find_element(*self._delete_locator).tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time
from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class Clock(Base):

    name = 'Clock'

    _alarm_create_new_locator = (By.ID, 'alarm-new')
    _analog_clock_display_locator = (By.ID, 'analog-clock')
    _digital_clock_display_locator = (By.ID, 'digital-clock')
    _clock_day_date_locator = (By.ID, 'clock-day-date')
    _all_alarms_locator = (By.CSS_SELECTOR, '#alarms li')
    _banner_countdown_notification_locator = (By.ID, 'banner-countdown')

    def launch(self):
        Base.launch(self)
        self.wait_for_new_alarm_button()
        # Desperate attempt to bust the intermittency :(
        time.sleep(1)

    @property
    def is_digital_clock_displayed(self):
        return self.is_element_displayed(*self._digital_clock_display_locator)

    @property
    def is_analog_clock_displayed(self):
        return self.is_element_displayed(*self._analog_clock_display_locator)

    @property
    def is_day_and_date_displayed(self):
        return self.is_element_displayed(*self._clock_day_date_locator)

    @property
    def banner_countdown_notification(self):
        return self.marionette.find_element(*self._banner_countdown_notification_locator).text

    @property
    def alarms(self):
        return [self.Alarm(self.marionette, alarm) for alarm in self.marionette.find_elements(*self._all_alarms_locator)]

    def wait_for_new_alarm_button(self):
        self.wait_for_element_displayed(*self._alarm_create_new_locator)

    def wait_for_banner_not_visible(self):
        self.wait_for_element_not_displayed(*self._banner_countdown_notification_locator)

    def wait_for_banner_displayed(self):
        self.wait_for_element_displayed(*self._banner_countdown_notification_locator)

    def tap_analog_display(self):
        self.wait_for_element_displayed(*self._analog_clock_display_locator)
        self.marionette.find_element(*self._analog_clock_display_locator).tap()
        self.wait_for_element_displayed(*self._digital_clock_display_locator)

    def tap_digital_display(self):
        self.wait_for_element_displayed(*self._digital_clock_display_locator)
        self.marionette.find_element(*self._digital_clock_display_locator).tap()
        self.wait_for_element_displayed(*self._analog_clock_display_locator)

    def tap_new_alarm(self):
        self.wait_for_element_displayed(*self._alarm_create_new_locator)
        self.marionette.find_element(*self._alarm_create_new_locator).tap()

        from gaiatest.apps.clock.regions.alarm import NewAlarm
        new_alarm = NewAlarm(self.marionette)
        new_alarm.wait_for_panel_to_load()
        return new_alarm

    class Alarm(PageRegion):

        _label_locator = (By.CSS_SELECTOR, '.label')
        _time_locator = (By.CSS_SELECTOR, '.time')
        _check_box_locator = (By.CSS_SELECTOR, '.input-enable')
        _enable_button_locator = (By.CSS_SELECTOR, '.alarmEnable')

        @property
        def label(self):
            return self.root_element.find_element(*self._label_locator).text

        @property
        def time(self):
            return self.root_element.find_element(*self._time_locator).text

        @property
        def is_alarm_active(self):
            return self.root_element.find_element(*self._check_box_locator).is_selected()

        def tap_checkbox(self):
            self.root_element.find_element(*self._enable_button_locator).tap()

        def tap(self):
            self.root_element.tap()
            from gaiatest.apps.clock.regions.alarm import EditAlarm
            return EditAlarm(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Battery(Base):

    _power_save_checkbox_locator = (By.CSS_SELECTOR, '.uninit[name*="powersave"]')
    _power_save_label_locator = (By.CSS_SELECTOR, 'span[data-l10n-id="powerSaveMode"]')

    def toggle_power_save_mode(self):
        checkbox = self.marionette.find_element(*self._power_save_checkbox_locator)
        label = self.marionette.find_element(*self._power_save_label_locator)
        checkbox_state = checkbox.is_selected()
        label.tap()
        self.wait_for_condition(lambda m: checkbox_state is not checkbox.is_selected())



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.cost_control.regions.ftu_step2 import FTUStep2


class FTUStep1(Base):

    _welcome_title_locator = (By.CSS_SELECTOR, 'h1[data-l10n-id="fte-welcome-title"]')
    _next_button_locator = (By.CSS_SELECTOR, '#step-1 span[data-l10n-id="next"]')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._welcome_title_locator)

    def tap_next(self):
        self.wait_for_element_displayed(*self._next_button_locator)
        self.marionette.find_element(*self._next_button_locator).tap()
        return FTUStep2(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class AppPermissions(Base):

    _section_locator = (By.ID, 'appPermissions')
    _apps_locator = (By.XPATH, "//section[@id='appPermissions']//li[a[text()='%s']]")

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        section = self.marionette.find_element(*self._section_locator)
        self.wait_for_condition(lambda m: section.location['x'] == 0)

    def app_locator(self, app):
        return (self._apps_locator[0], self._apps_locator[1] % app)

    def tap_app(self, app):
        self.marionette.find_element(*self.app_locator(app)).tap()
        return AppPermissionsDetails(self.marionette)


class AppPermissionsDetails(Base):

    _section_locator = (By.ID, 'appPermissions-details')
    _permissions_locator = (By.CSS_SELECTOR, 'p[data-l10n-id="perm-geolocation"]')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        section = self.marionette.find_element(*self._section_locator)
        self.wait_for_condition(lambda m: section.location['x'] == 0)

    @property
    def is_geolocation_listed(self):
        return self.marionette.find_element(*self._permissions_locator).is_displayed()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time


class MockEmail(dict):

    def __init__(self, senders_email, recipients_email, **kwargs):
        current_time = time.time()
        self['subject'] = 'Test email subject %s' % current_time
        self['message'] = 'Test email message %s' % current_time
        self['recipients_email'] = recipients_email
        self['senders_email'] = senders_email

        self.update(**kwargs)

    # allow getting items as if they were attributes
    def __getattr__(self, attr):
        return self[attr]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Wifi(Base):

    _wifi_enabled_label_locator = (By.CSS_SELECTOR, '#wifi-enabled label')
    _wifi_enabled_checkbox_locator = (By.CSS_SELECTOR, '#wifi-enabled input')
    _available_networks_locator = (By.CSS_SELECTOR, '#wifi-availableNetworks > li > aside[class*="wifi-signal"]')
    _password_input_locator = (By.CSS_SELECTOR, '#wifi-auth input[type="password"]')
    _password_ok_button_locator = (By.CSS_SELECTOR, '#wifi-auth button[type="submit"]')
    _connected_message_locator = (By.CSS_SELECTOR, '#wifi-availableNetworks li.active small')

    @property
    def is_wifi_enabled(self):
        return self.marionette.find_element(*self._wifi_enabled_checkbox_locator).get_attribute('checked')

    def enable_wifi(self):
        self.marionette.find_element(*self._wifi_enabled_label_locator).tap()
        self.wait_for_condition(lambda m: self.is_wifi_enabled)

    def connect_to_network(self, network_info):
        # Wait for some networks to be found
        self.wait_for_condition(lambda m: len(m.find_elements(*self._available_networks_locator)) > 0,
                                message="No networks listed on screen")

        this_network_locator = ('xpath', "//li/a[text()='%s']" % network_info['ssid'])
        self.marionette.find_element(*this_network_locator).tap()

        if network_info.get('keyManagement'):
            password = network_info.get('psk') or network_info.get('wep')
            if not password:
                raise Exception('No psk or wep key found in testvars for secured wifi network.')

            self.wait_for_element_displayed(*self._password_input_locator)
            password_input = self.marionette.find_element(*self._password_input_locator)
            password_input.send_keys(password)
            self.marionette.find_element(*self._password_ok_button_locator).tap()

        self.wait_for_condition(
            lambda m: m.find_element(*self._connected_message_locator).text == "Connected",
                        timeout = max(self.marionette.timeout and self.marionette.timeout / 1000, 60))



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time

from marionette.by import By
from marionette.marionette import Actions

from gaiatest.apps.base import Base


class CardsView(Base):

    # Home/Cards view locators
    _cards_view_locator = (By.CSS_SELECTOR, '#cards-view.active')
    _no_apps_locator = (By.CSS_SELECTOR, '.no-recent-apps')
    # Check that the origin contains the current app name, origin is in the format:
    # app://clock.gaiamobile.org
    _apps_cards_locator = (By.CSS_SELECTOR, '#cards-view li[data-origin*="%s"]')
    _close_buttons_locator = (By.CSS_SELECTOR, '#cards-view li[data-origin*="%s"] .close-card')

    def _app_card_locator(self, app):
        return (self._apps_cards_locator[0], self._apps_cards_locator[1] % app.lower())

    def _close_button_locator(self, app):
        return (self._close_buttons_locator[0], self._close_buttons_locator[1] % app.lower())

    def open_cards_view(self):
        # Hold the home button to open cards view
        self.marionette.execute_script("window.wrappedJSObject.dispatchEvent(new Event('holdhome'));")
        time.sleep(3)
        self.wait_for_cards_view()

    def exit_cards_view(self):
        # Touch the home button to exit cards view
        self.marionette.execute_script("window.wrappedJSObject.dispatchEvent(new Event('home'));")
        self.wait_for_cards_view_not_displayed()

    @property
    def is_cards_view_displayed(self):
        return self.is_element_displayed(*self._cards_view_locator)

    def is_app_displayed(self, app):
        return self.marionette.find_element(*self._app_card_locator(app)).is_displayed()

    def is_app_present(self, app):
        return self.is_element_present(*self._app_card_locator(app))

    def tap_app(self, app):
        return self.marionette.find_element(*self._app_card_locator(app)).tap()

    def close_app(self, app):
        return self.marionette.find_element(*self._close_button_locator(app)).tap()

    def wait_for_cards_view(self):
        self.wait_for_element_displayed(*self._cards_view_locator)

    def wait_for_cards_view_not_displayed(self):
        self.wait_for_element_not_displayed(*self._cards_view_locator)

    @property
    def no_recent_apps_message(self):
        return self.marionette.find_element(*self._no_apps_locator).text

    def swipe_to_next_app(self):
        current_frame = self.apps.displayed_app.frame

        start_x_position = current_frame.size['width']
        start_y_position = current_frame.size['height'] // 2

        # swipe backward to get next app card
        Actions(self.marionette).flick(
            current_frame, start_x_position, start_y_position, 0, start_y_position).perform()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import json


class MockContact(dict):
    '''
    The key values here match the data structure in the contacts db
    so that the json output of this mock can be inserted directly into db
    '''

    def __init__(self, **kwargs):
        # set your default values
        import time
        curr_time = repr(time.time()).replace('.', '')
        self['givenName'] = ['gaia%s' % curr_time[10:]]
        self['familyName'] = ['test']
        self['name'] = [self['givenName'][0] + " " + self['familyName'][0]]
        self['email'] = [{
            'type': ['Personal'],
            'value': '%s@restmail.net' % self['givenName'][0]}]
        # TODO this will only support one phone number
        self['tel'] = [{
            'type': ['Mobile'],
            'value': '555%s' % curr_time[8:]}]
        self['adr'] = [{
            'type': ['Home'],
            'streetAddress': '101 Testing street',
            'postalCode': '90210',
            'locality': 'London',
            'countryName': 'UK'}]
        self['note'] = ["Gaia automated test"]

        # update with any keyword arguments passed
        self.update(**kwargs)

    # allow getting items as if they were attributes
    def __getattr__(self, attr):
        return self[attr]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.phone.app import Phone


class AttentionScreen(Phone):

    _attention_frame_locator = (By.CSS_SELECTOR, 'iframe[src^="app://communications"][src$="index.html#keyboard-view"]')
    _message_locator = (By.ID, 'message')

    def __init__(self, marionette):
        Phone.__init__(self, marionette)
        self.marionette.switch_to_frame()
        self.marionette.switch_to_frame(self.marionette.find_element(*self._attention_frame_locator))

    @property
    def message(self):
        return self.marionette.find_element(*self._message_locator).text



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class PhoneLock(Base):

    _phone_lock_section_locator = (By.ID, 'phoneLock')
    _passcode_enable_locator = (By.CSS_SELECTOR, 'li.lockscreen-enabled label')
    _phone_lock_passcode_section_locator = (By.ID, 'phoneLock-passcode')
    _passcode_create_locator = (By.ID, 'passcode-create')

    def enable_passcode_lock(self):
        self.marionette.find_element(*self._passcode_enable_locator).tap()
        self.wait_for_element_displayed(*self._phone_lock_passcode_section_locator)

    def create_passcode(self, passcode):

        # switch to keyboard, input passcode
        for times in range(2):
            self.keyboard.send("".join(passcode))

        # switch to settings frame
        self.marionette.switch_to_frame()
        self.marionette.switch_to_frame(self.apps.displayed_app.frame)

        # create passcode
        self.wait_for_element_displayed(*self._phone_lock_passcode_section_locator)
        self.marionette.find_element(*self._passcode_create_locator).tap()
        self.wait_for_element_displayed(*self._phone_lock_section_locator)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By

from gaiatest.apps.base import Base
from gaiatest.apps.music.regions.list_view import ListView


class Music(Base):

    name = 'Music'

    _loading_spinner_locator = (By.ID, 'spinner-overlay')
    _empty_music_title_locator = (By.ID, 'overlay-title')
    _empty_music_text_locator = (By.ID, 'overlay-text')
    _albums_tab_locator = (By.ID, 'tabs-albums')

    def launch(self):
        Base.launch(self)
        self.wait_for_element_not_displayed(*self._loading_spinner_locator)

    def wait_for_empty_message_to_load(self):
        empty_title = self.marionette.find_element(*self._empty_music_title_locator)
        self.wait_for_condition(lambda m: empty_title.text != '')

    @property
    def empty_music_title(self):
        return self.marionette.find_element(*self._empty_music_title_locator).text

    @property
    def empty_music_text(self):
        return self.marionette.find_element(*self._empty_music_text_locator).text

    def tap_albums_tab(self):
        self.wait_for_element_displayed(*self._albums_tab_locator)
        self.marionette.find_element(*self._albums_tab_locator).tap()
        return ListView(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.cost_control.app import CostControl
from gaiatest.apps.cost_control.regions.ftu_step3 import FTUStep3


class FTUStep2(CostControl):

    _reset_report_menu_locator = (By.CSS_SELECTOR, '#non-vivo-step-1 menu')
    _reset_report_period_select_locator = (By.CSS_SELECTOR, '#non-vivo-step-1 ul li:nth-child(1) span')
    _next_button_locator = (By.CSS_SELECTOR, '#non-vivo-step-1 span[data-l10n-id="next"]')

    def __init__(self, marionette):
        CostControl.__init__(self, marionette)
        menu = self.marionette.find_element(*self._reset_report_menu_locator)
        self.wait_for_condition(lambda m: menu.location['x'] == 0)

    def select_reset_report_value(self, value):
        reset_time = self.marionette.find_element(*self._reset_report_period_select_locator)
        reset_time.tap()
        self.select(value)
        self.switch_to_ftu()

    def tap_next(self):
        self.marionette.find_element(*self._next_button_locator).tap()
        return FTUStep3(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Keyboard(Base):

    _section_locator = (By.ID, 'keyboard')
    _selected_keyboards_link_locator = (By.CSS_SELECTOR, "a[data-l10n-id='selectedKeyboards']")

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        section = self.marionette.find_element(*self._section_locator)
        self.wait_for_condition(lambda m: section.location['x'] == 0)

    def tap_selected_keyboards(self):
        self.marionette.find_element(*self._selected_keyboards_link_locator).tap()
        return KeyboardSelectKeyboard(self.marionette)


class KeyboardSelectKeyboard(Base):

    _section_locator = (By.ID, 'keyboard-selection')
    _add_more_keyboards_button_locator = (By.CSS_SELECTOR, "button[data-l10n-id='addMoreKeyboards']")

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        section = self.marionette.find_element(*self._section_locator)
        self.wait_for_condition(lambda m: section.location['x'] == 0)

    def tap_add_more_keyboards(self):
        self.marionette.find_element(*self._add_more_keyboards_button_locator).tap()
        return KeyboardAddMoreKeyboards(self.marionette)


class KeyboardAddMoreKeyboards(Base):

    _section_locator = (By.ID, 'keyboard-selection-addMore')
    _select_language_locator = (
        By.XPATH,
        "//div[@id='keyboardAppContainer']//li[label[span[text()='%s']]]"
    )

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        section = self.marionette.find_element(*self._section_locator)
        self.wait_for_condition(lambda m: section.location['x'] == 0)

    def select_language(self, language):
        language_locator = (
            self._select_language_locator[0],
            self._select_language_locator[1] % language
        )
        self.wait_for_element_displayed(*language_locator)
        selected_language = self.marionette.find_element(*language_locator)
        # TODO bug 878017 - remove the explicit scroll once bug is fixed
        # We still need this unfortunately
        self.marionette.execute_script("arguments[0].scrollIntoView(false);", [selected_language])
        selected_language.tap()
        checkbox = selected_language.find_element(By.TAG_NAME, 'input')
        self.wait_for_condition(lambda m: checkbox.is_selected())



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion
from gaiatest.apps.messages.app import Messages


class MessageThread(Base):

    _all_messages_locator = (By.CSS_SELECTOR, '#messages-container li.message')
    _received_message_content_locator = (By.CSS_SELECTOR, "#messages-container li.message.received")
    _back_header_link_locator = (By.ID, 'messages-back-button')
    _message_header_locator = (By.ID, 'messages-header-text')
    _call_button_locator = (By.CSS_SELECTOR, 'button[data-l10n-id="call"]')

    def wait_for_received_messages(self, timeout=180):
        self.wait_for_element_displayed(*self._received_message_content_locator, timeout=timeout)

    def tap_back_button(self):
        # In a message thread, tap the back button to return to main message list
        back_header_button = self.marionette.find_element(*self._back_header_link_locator)
        back_header_button.tap()
        messages = Messages(self.marionette)
        messages.wait_for_message_list()
        return messages

    @property
    def received_messages(self):
        return [Message(self.marionette, message) for message in self.marionette.find_elements(*self._received_message_content_locator)]

    @property
    def all_messages(self):
        return [Message(self.marionette, message) for message in self.marionette.find_elements(*self._all_messages_locator)]

    def tap_header(self):
        self.wait_for_element_displayed(*self._message_header_locator)
        self.marionette.find_element(*self._message_header_locator).tap()

    def tap_call(self):
        self.marionette.find_element(*self._call_button_locator).tap()
        self.wait_for_element_not_displayed(*self._call_button_locator)
        from gaiatest.apps.phone.regions.keypad import Keypad
        keypad = Keypad(self.marionette)
        keypad.switch_to_keypad_frame()
        return keypad


class Message(PageRegion):

    _text_locator = (By.CSS_SELECTOR, '.bubble > p')
    _attachments_locator = (By.CSS_SELECTOR, '.bubble .attachment-container.preview')

    @property
    def text(self):
        return self.root_element.find_element(*self._text_locator).text

    @property
    def has_attachments(self):
        try:
            self.root_element.find_element(*self._attachments_locator)
        except:
            return False

        return True

    @property
    def id(self):
        return self.root_element.get_attribute('id')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import time
from marionette.by import By
from gaiatest.apps.base import Base


class FullscreenVideo(Base):

    _video_controls_locator = (By.ID, 'videoControls')
    _video_title_locator = (By.ID, 'video-title')
    _elapsed_text_locator = (By.ID, 'elapsed-text')
    _video_player_locator = (By.ID, 'player')
    _video_frame_locator = (By.CSS_SELECTOR, "iframe[src^='app://video'][src$='view.html']")
    _video_player_frame_locator = (By.ID, 'fullscreen-view')
    _spinner_overlay_locator = (By.ID, 'spinner-overlay')

    def wait_for_player_frame_displayed(self):
        self.wait_for_element_displayed(*self._video_player_frame_locator)

    def display_controls_with_js(self):
        self.marionette.execute_script("window.wrappedJSObject.showVideoControls(true);")
        self.wait_for_element_displayed(*self._video_controls_locator)

    @property
    def elapsed_time(self):
        # Convert it to a real time so we can accurately assert
        text = self.marionette.find_element(*self._elapsed_text_locator).text
        return time.strptime(text, '%M:%S')

    @property
    def name(self):
        return self.marionette.find_element(*self._video_title_locator).text

    @property
    def is_video_playing(self):
        return self.marionette.find_element(*self._video_player_locator).get_attribute('paused') == 'false'

    def switch_to_video_frame(self):
        self.marionette.switch_to_frame(self.marionette.find_element(*self._video_frame_locator))
        self.wait_for_element_not_displayed(*self._spinner_overlay_locator)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from marionette.marionette import Actions

from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class Collection(Base):

    _apps_locator = (By.CSS_SELECTOR, '.evme-apps ul.cloud li[data-name]')
    _homescreen_status_notification = (By.CSS_SELECTOR, "section[role='status'] > p")
    _close_collection_locator = (By.CSS_SELECTOR, '#collection div.header .close')

    def wait_for_collection_screen_visible(self):
        self.wait_for_element_displayed(*self._apps_locator)

    def tap_exit(self):
        self.marionette.find_element(*self._close_collection_locator).tap()
        self.wait_for_element_not_displayed(*self._close_collection_locator)
        from gaiatest.apps.homescreen.app import Homescreen
        return Homescreen(self.marionette)

    @property
    def notification_message(self):
        self.wait_for_element_displayed(*self._homescreen_status_notification)
        return self.marionette.find_element(*self._homescreen_status_notification).text

    @property
    def applications(self):
        return [self.Result(self.marionette, app) for app in self.marionette.find_elements(*self._apps_locator)]

    class Result(PageRegion):

        # Modal dialog locators
        _modal_dialog_save_locator = (By.CSS_SELECTOR, ".cloud-app-actions.show > menu > button[data-action = 'save']")

        @property
        def name(self):
            return self.root_element.get_attribute('data-name')

        def tap(self):
            app_name = self.name

            self.root_element.tap()
            # Wait for the displayed app to be that we have tapped
            self.wait_for_condition(lambda m: self.apps.displayed_app.name == app_name)
            self.marionette.switch_to_frame(self.apps.displayed_app.frame)

            # Wait for title to load (we cannot be more specific because the aut may change)
            self.wait_for_condition(lambda m: m.title)

        def long_tap_to_install(self):
            Actions(self.marionette).long_press(self.root_element, 2).perform()

        def tap_save_to_home_screen(self):
            self.wait_for_element_displayed(*self._modal_dialog_save_locator)
            self.marionette.find_element(*self._modal_dialog_save_locator).tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By

from gaiatest.apps.base import Base, PageRegion
from gaiatest.apps.music.regions.sublist_view import SublistView


class ListView(Base):

    _album_list_item_locator = (By.CSS_SELECTOR, '.list-item')

    @property
    def albums(self):
        self.wait_for_element_displayed(*self._album_list_item_locator)
        return [Album(self.marionette, album) for album in
                self.marionette.find_elements(*self._album_list_item_locator)]


class Album(PageRegion):

    _album_link_locator = (By.TAG_NAME, 'a')

    def tap(self):
        self.marionette.find_element(*self._album_link_locator).tap()
        return SublistView(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from marionette.marionette import Actions

from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class Homescreen(Base):

    name = 'Homescreen'

    _homescreen_icon_locator = (By.CSS_SELECTOR, 'li.icon[aria-label="%s"]')
    _visible_icons_locator = (By.CSS_SELECTOR, 'div.page[style*="transform: translateX(0px);"] > ol > .icon')
    _edit_mode_locator = (By.CSS_SELECTOR, 'body[data-mode="edit"]')
    _search_bar_icon_locator = (By.CSS_SELECTOR, '#evme-activation-icon input')
    _landing_page_locator = (By.ID, 'icongrid')
    _collections_locator = (By.CSS_SELECTOR, 'li.icon[data-collection-name]')
    _collection_locator = (By.CSS_SELECTOR, "li.icon[data-collection-name *= '%s']")

    def launch(self):
        Base.launch(self)

    def tap_search_bar(self):
        search_bar = self.marionette.find_element(*self._search_bar_icon_locator)
        search_bar.tap()
        from gaiatest.apps.homescreen.regions.search_panel import SearchPanel
        return SearchPanel(self.marionette)

    def wait_for_app_icon_present(self, app_name):
        self.wait_for_element_present(self._homescreen_icon_locator[0], self._homescreen_icon_locator[1] % app_name)

    def wait_for_app_icon_not_present(self, app_name):
        self.wait_for_element_not_present(self._homescreen_icon_locator[0], self._homescreen_icon_locator[1] % app_name)

    def is_app_installed(self, app_name):
        """Checks whether app is installed"""
        is_installed = False
        for i in range(self.homescreen_get_total_pages_number):
            if self.is_element_displayed(self._homescreen_icon_locator[0], self._homescreen_icon_locator[1] % app_name):
                is_installed = True
                break
            elif self.homescreen_has_more_pages:
                self.go_to_next_page()

        return is_installed

    def go_to_next_page(self):
        self.marionette.execute_script('window.wrappedJSObject.GridManager.goToNextPage()')
        self.wait_for_condition(lambda m: m.find_element('tag name', 'body')
            .get_attribute('data-transitioning') != 'true')

    def touch_home_button(self):
        self.marionette.switch_to_frame()
        self.marionette.execute_script("window.wrappedJSObject.dispatchEvent(new Event('home'));")
        self.wait_for_condition(lambda m: self.apps.displayed_app.name == self.name)
        self.marionette.switch_to_frame(self.apps.displayed_app.frame)

    def activate_edit_mode(self):
        app = self.marionette.find_element(*self._visible_icons_locator)
        Actions(self.marionette).\
            press(app).\
            wait(3).\
            release().\
            perform()
        self.wait_for_element_displayed(By.CSS_SELECTOR, 'div.dockWrapper ol[style*="transition: -moz-transform 0.5ms ease 0s;"]')

    def open_context_menu(self):
        test = self.marionette.find_element(*self._landing_page_locator)
        Actions(self.marionette).\
            press(test, 0, 0).\
            wait(3).\
            release().\
            perform()
        from gaiatest.apps.homescreen.regions.context_menu import ContextMenu
        return ContextMenu(self.marionette)

    def move_app_to_position(self, app_position, to_position):
        app = self.marionette.find_elements(*self._visible_icons_locator)[app_position]
        destination = self.marionette.find_elements(*self._visible_icons_locator)[to_position]

        Actions(self.marionette).\
            press(app).\
            wait(3).\
            move(destination).\
            wait(1).\
            release().\
            perform()

    @property
    def is_edit_mode_active(self):
        return self.is_element_present(*self._edit_mode_locator)

    @property
    def homescreen_get_total_pages_number(self):
        return self.marionette.execute_script("""
        var pageHelper = window.wrappedJSObject.GridManager.pageHelper;
        return pageHelper.getTotalPagesNumber();""")

    @property
    def homescreen_has_more_pages(self):
        # the naming of this could be more concise when it's in an app object!
        return self.marionette.execute_script("""
        var pageHelper = window.wrappedJSObject.GridManager.pageHelper;
        return pageHelper.getCurrentPageNumber() < (pageHelper.getTotalPagesNumber() - 1);""")

    @property
    def collections_count(self):
        return len(self.marionette.find_elements(*self._collections_locator))

    def tap_collection(self, name):
        el = self.marionette.find_element(self._collection_locator[0],
                                          self._collection_locator[1] % name)
        el.tap()

        from gaiatest.apps.homescreen.regions.collections import Collection
        return Collection(self.marionette)

    @property
    def visible_apps(self):
        return [self.InstalledApp(self.marionette, root_el)
                for root_el in self.marionette.find_elements(*self._visible_icons_locator)]

    def installed_app(self, app_name):
        root_el = self.marionette.find_element(self._homescreen_icon_locator[0], self._homescreen_icon_locator[1] % app_name)
        return self.InstalledApp(self.marionette, root_el)

    class InstalledApp(PageRegion):

        _delete_app_locator = (By.CSS_SELECTOR, 'li.icon[aria-label="%s"] span.options')

        @property
        def name(self):
            return self.root_element.get_attribute('aria-label')

        def tap_icon(self):
            expected_name = self.name
            self.root_element.tap()
            self.wait_for_condition(lambda m: self.apps.displayed_app.name.lower() == expected_name.lower())
            self.marionette.switch_to_frame(self.apps.displayed_app.frame)

        def tap_delete_app(self):
            """Tap on (x) to delete app"""
            delete_app_locator = (self._delete_app_locator[0], self._delete_app_locator[1] % self.name)
            self.wait_for_element_displayed(*delete_app_locator)
            self.marionette.find_element(*delete_app_locator).tap()

            from gaiatest.apps.homescreen.regions.confirm_dialog import ConfirmDialog
            return ConfirmDialog(self.marionette)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.


class MockUser(dict):

    def __init__(self, **kwargs):
        # set your default values
        import time
        self['name'] = 'marketplace_%s' % repr(time.time()).replace('.', '')
        self['email'] = '%s@restmail.net' % self['name']
        self['password'] = 'Password12345'

        # update with any keyword arguments passed
        self.update(**kwargs)

    # allow getting items as if they were attributes
    def __getattr__(self, attr):
        return self[attr]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class ConfirmInstall(Base):

        _confirm_install_button_locator = (By.ID, 'app-install-install-button')

        def tap_confirm(self):
            self.wait_for_element_displayed(*self._confirm_install_button_locator)
            self.marionette.find_element(*self._confirm_install_button_locator).tap()
            self.wait_for_element_not_displayed(*self._confirm_install_button_locator)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base

import gaiatest.apps.camera.app


class Gallery(Base):

    name = 'Gallery'

    _gallery_frame_locator = (By.CSS_SELECTOR, "iframe[src^='app://gallery'][src$='index.html#pick']")
    _gallery_items_locator = (By.CSS_SELECTOR, 'li.thumbnail')
    _empty_gallery_title_locator = (By.ID, 'overlay-title')
    _empty_gallery_text_locator = (By.ID, 'overlay-text')
    _progress_bar_locator = (By.ID, 'progress')
    _thumbnail_list_view_locator = (By.ID, 'thumbnail-list-view')
    _switch_to_camera_button_locator = (By.ID, 'thumbnails-camera-button')

    def launch(self):
        Base.launch(self)
        self.wait_for_element_not_displayed(*self._progress_bar_locator)
        self.wait_for_element_displayed(*self._thumbnail_list_view_locator)

    def switch_to_gallery_frame(self):
        self.wait_for_element_displayed(*self._gallery_frame_locator)
        self.marionette.switch_to_frame(self.marionette.find_element(*self._gallery_frame_locator))

    def wait_for_files_to_load(self, files_number):
        self.wait_for_condition(lambda m: m.execute_script('return window.wrappedJSObject.files.length') == files_number)

    def wait_for_thumbnails_to_load(self):
        self.wait_for_element_displayed(*self._gallery_items_locator)

    @property
    def gallery_items_number(self):
        return len(self.marionette.find_elements(*self._gallery_items_locator))

    def tap_first_gallery_item(self):
        first_gallery_item = self.marionette.find_elements(*self._gallery_items_locator)[0]
        if self.is_element_displayed(*self._thumbnail_list_view_locator):
            from gaiatest.apps.gallery.regions.fullscreen_image import FullscreenImage as NextView
        else:
            from gaiatest.apps.gallery.regions.crop_view import CropView as NextView
        first_gallery_item.tap()
        return NextView(self.marionette)

    @property
    def empty_gallery_title(self):
        return self.marionette.find_element(*self._empty_gallery_title_locator).text

    @property
    def empty_gallery_text(self):
        return self.marionette.find_element(*self._empty_gallery_text_locator).text

    @property
    def are_gallery_items_displayed(self):
        return self.marionette.find_element(*self._gallery_items_locator).is_displayed()

    def switch_to_camera(self):
        switch_to_camera_button = self.marionette.find_element(*self._switch_to_camera_button_locator)
        switch_to_camera_button.tap()
        camera_app = gaiatest.apps.camera.app.Camera(self.marionette)
        self.wait_for_condition(lambda m: self.apps.displayed_app.name == camera_app.name)
        self.marionette.switch_to_frame(self.apps.displayed_app.frame)
        return camera_app



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


UI_TESTS = "UI Tests"


class UiTests(Base):
    _ui_page_locator = (By.CSS_SELECTOR, 'a[href="#UI"]')
    _api_page_locator = (By.CSS_SELECTOR, 'a[href="#API"]')
    _hw_page_locator = (By.CSS_SELECTOR, 'a[href="#HW"]')

    _moz_id_tests_button_locator = (By.LINK_TEXT, 'navigator.mozId')
    _keyboard_locator = (By.LINK_TEXT, 'Keyboard')
    _context_menu_locator = (By.LINK_TEXT, 'Contextmenu')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.name = UI_TESTS

    def launch(self):
        Base.launch(self, launch_timeout=120000)

    def tap_ui_button(self):
        self.wait_for_element_displayed(*self._ui_page_locator)
        self.marionette.find_element(*self._ui_page_locator).tap()

    def tap_api_button(self):
        self.wait_for_element_displayed(*self._api_page_locator)
        self.marionette.find_element(*self._api_page_locator).tap()

    def tap_hw_button(self):
        self.wait_for_element_displayed(*self._hw_page_locator)
        self.marionette.find_element(*self._hw_page_locator).tap()

    def tap_moz_id_button(self):
        self.wait_for_element_displayed(*self._moz_id_tests_button_locator, timeout=120)
        # Hack to make the identity button visible from underneath the toolbar
        mozId_button = self.marionette.find_element(*self._moz_id_tests_button_locator)
        self.marionette.execute_script('arguments[0].scrollIntoView(false);', [mozId_button])
        mozId_button.tap()

        from gaiatest.apps.ui_tests.regions.persona import Persona

        return Persona(self.marionette)

    def tap_keyboard_option(self):
        self.wait_for_element_displayed(*self._keyboard_locator, timeout=120)
        # Hack to make the keyboard button visible from underneath the toolbar
        keyboard_button = self.marionette.find_element(*self._keyboard_locator)
        self.marionette.execute_script('arguments[0].scrollIntoView(false);', [keyboard_button])
        keyboard_button.tap()

        from gaiatest.apps.ui_tests.regions.keyboard import KeyboardPage

        return KeyboardPage(self.marionette)

    def tap_context_menu_option(self):
        self.wait_for_element_displayed(*self._context_menu_locator, timeout=120)
        self.marionette.find_element(*self._context_menu_locator).tap()

        from gaiatest.apps.ui_tests.regions.context_menu import ContextMenuPage

        return ContextMenuPage(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.settings.regions.cell_data_prompt import CellDataPrompt


class CellData(Base):

    _carrier_name_locator = (By.ID, 'dataNetwork-desc')
    _cell_data_enabled_input_locator = (By.XPATH, "//input[@name='ril.data.enabled']")
    _cell_data_enabled_label_locator = (By.XPATH, "//input[@name='ril.data.enabled']/..")

    @property
    def carrier_name(self):
        self.wait_for_element_displayed(*self._carrier_name_locator)
        return self.marionette.find_element(*self._carrier_name_locator).text

    @property
    def is_data_enabled(self):
        return self.marionette.find_element(*self._cell_data_enabled_input_locator).get_attribute('checked')

    def enable_data(self):
        self.wait_for_element_displayed(*self._cell_data_enabled_label_locator)
        self.marionette.find_element(*self._cell_data_enabled_label_locator).tap()
        return CellDataPrompt(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Settings(Base):

    _settings_iframe_locator = (By.ID, 'settings-view-placeholder')
    _settings_title_locator = (By.CSS_SELECTOR, 'section#settings-view h1')

    _data_alert_label_locator = (By.XPATH, "//ul[preceding-sibling::header[@id='data-usage-settings']]/li[2]/label")
    _data_alert_switch_locator = (By.CSS_SELECTOR, 'input[data-option="dataLimit"]')
    _when_use_is_above_button_locator = (By.CSS_SELECTOR, 'button[data-widget-type="data-limit"]')
    _unit_button_locator = (By.CSS_SELECTOR, '#data-limit-dialog form button')
    _size_input_locator = (By.CSS_SELECTOR, '#data-limit-dialog form input')
    _usage_done_button_locator = (By.ID, 'data-usage-done-button')

    _reset_button_locator = (By.ID, 'reset-data-usage')
    _reset_wifi_usage_button_locator = (By.ID, 'reset-data-wifi-usage')
    _done_button_locator = (By.CSS_SELECTOR, 'section#settings-view button#close-settings')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        # go into iframe of usage app settings
        self.wait_for_element_displayed(*self._settings_iframe_locator)
        self.marionette.switch_to_frame(self.marionette.find_element(*self._settings_iframe_locator))
        self.wait_for_element_displayed(*self._settings_title_locator)

    def toggle_data_alert_switch(self, value):
        self.wait_for_element_displayed(*self._data_alert_label_locator)
        switch = self.marionette.find_element(*self._data_alert_switch_locator)
        if switch.is_selected() != value:
            label = self.marionette.find_element(*self._data_alert_label_locator)
            label.tap()

    def select_when_use_is_above_unit_and_value(self, unit, value):

        when_use_is_above_button = self.marionette.find_element(*self._when_use_is_above_button_locator)
        self.wait_for_condition(
            lambda m: m.find_element(*self._when_use_is_above_button_locator).get_attribute('disabled') == 'false'
        )
        when_use_is_above_button.tap()

        self.wait_for_element_displayed(*self._unit_button_locator)
        current_unit = self.marionette.find_element(*self._unit_button_locator)
        if current_unit.text != unit:
            current_unit.tap()
            # We need to wait for the javascript to do its stuff
            self.wait_for_condition(lambda m: m.find_element(*self._unit_button_locator).text == unit)

        # clear the original assigned value and set it to the new value
        self.wait_for_element_displayed(*self._size_input_locator)
        size = self.marionette.find_element(*self._size_input_locator)
        size.clear()
        size.send_keys(value)
        done = self.marionette.find_element(*self._usage_done_button_locator)
        done.tap()

    def reset_data_usage(self):
        self.wait_for_element_displayed(*self._settings_title_locator)
        self.marionette.find_element(*self._reset_button_locator).tap()
        self.wait_for_element_displayed(*self._reset_wifi_usage_button_locator)
        self.marionette.find_element(*self._reset_wifi_usage_button_locator).tap()
        self.wait_for_element_displayed(*self._settings_title_locator)

    def tap_done(self):
        self.wait_for_element_displayed(*self._done_button_locator)
        self.marionette.find_element(*self._done_button_locator).tap()
        # Switch back to Cost Control app frame
        self.apps.switch_to_displayed_app()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class DeviceInfo(Base):

    _phone_number_locator = (By.ID, 'deviceInfo-msisdn')
    _model_locator = (By.CSS_SELECTOR, '#about small[data-name="deviceinfo.hardware"]')
    _software_locator = (By.CSS_SELECTOR, '#about small[data-name="deviceinfo.software"]')
    _more_info_button_locator = (By.CSS_SELECTOR, '#about button[data-l10n-id="more-info"]')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_element_displayed(*self._phone_number_locator)

    @property
    def phone_number(self):
        return self.marionette.find_element(*self._phone_number_locator).text

    @property
    def model(self):
        return self.marionette.find_element(*self._model_locator).text

    @property
    def software(self):
        return self.marionette.find_element(*self._software_locator).text

    def tap_more_info(self):
        self.marionette.find_element(*self._more_info_button_locator).tap()
        return self.MoreInfo(self.marionette)

    class MoreInfo(Base):

        _os_version_locator = (By.CSS_SELECTOR, '#about-moreInfo small[data-name="deviceinfo.os"]')
        _hardware_revision_locator = (By.CSS_SELECTOR, '#about-moreInfo small[data-name="deviceinfo.hardware"]')
        _mac_address_locator = (By.CSS_SELECTOR, '#about-moreInfo small[data-name="deviceinfo.mac"]')
        _imei_locator = (By.ID, 'deviceInfo-imei')
        _iccid_locator = (By.ID, 'deviceInfo-iccid')
        _platform_version_locator = (By.CSS_SELECTOR, '#about-moreInfo small[data-name="deviceinfo.platform_version"]')
        _build_id_locator = (By.CSS_SELECTOR, '#about-moreInfo small[data-name="deviceinfo.platform_build_id"]')
        _update_channel_locator = (By.CSS_SELECTOR, '#about-moreInfo small[data-name="deviceinfo.update_channel"]')
        _git_commit_timestamp_locator = (By.ID, 'gaia-commit-date')
        _git_commit_hash_locator = (By.ID, 'gaia-commit-hash')

        def __init__(self, marionette):
            Base.__init__(self, marionette)
            self.wait_for_element_displayed(*self._os_version_locator)

        @property
        def os_version(self):
            return self.marionette.find_element(*self._os_version_locator).text

        @property
        def hardware_revision(self):
            return self.marionette.find_element(*self._hardware_revision_locator).text

        @property
        def mac_address(self):
            return self.marionette.find_element(*self._mac_address_locator).text

        @property
        def imei(self):
            return self.marionette.find_element(*self._imei_locator).text

        @property
        def iccid(self):
            return self.marionette.find_element(*self._iccid_locator).text

        @property
        def platform_version(self):
            return self.marionette.find_element(*self._platform_version_locator).text

        @property
        def build_id(self):
            return self.marionette.find_element(*self._build_id_locator).text

        @property
        def update_channel(self):
            return self.marionette.find_element(*self._update_channel_locator).text

        @property
        def git_commit_timestamp(self):
            return self.marionette.find_element(*self._git_commit_timestamp_locator).text

        @property
        def git_commit_hash(self):
            return self.marionette.find_element(*self._git_commit_hash_locator).text



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class ConfirmDialog(Base):

    _confirm_button_locator = (By.ID, 'confirm-dialog-confirm-button')

    def tap_confirm(self):
        self.wait_for_element_displayed(*self._confirm_button_locator)
        self.marionette.find_element(*self._confirm_button_locator).tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class EmergencyCallScreen(Base):

    _emergency_frame_locator = (By.CSS_SELECTOR, 'iframe[src*="/emergency-call/"]')
    _emergency_dialer_keypad_locator = (By.ID, 'keypad')

    def switch_to_emergency_call_screen(self):
        self.marionette.switch_to_frame()
        emergency_frame = self.marionette.find_element(*self._emergency_frame_locator)
        self.marionette.switch_to_frame(emergency_frame)
        self.wait_for_condition(
            lambda m: m.execute_script('return document.title') == 'Emergency Call Dialer')

    @property
    def is_emergency_dialer_keypad_displayed(self):
        return self.is_element_displayed(*self._emergency_dialer_keypad_locator)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Persona(Base):
    _app_ready_event = (By.CSS_SELECTOR, 'li.ready')
    _app_login_event = (By.CSS_SELECTOR, 'li.login')
    _app_logout_event = (By.CSS_SELECTOR, 'li.logout')
    _app_login_assertion_text = (By.CSS_SELECTOR, 'li.login div.assertion')
    _app_std_request_button_locator = (By.ID, 't-request')
    _app_logout_button_locator = (By.ID, 't-logout')

    _frame_locator = (By.CSS_SELECTOR, 'iframe[src*="identity"]')

    def launch_standard_sign_in(self):
        self.tap_standard_button()
        from gaiatest.apps.persona.app import Persona

        return Persona(self.marionette)

    def switch_to_frame(self):
        self.wait_for_element_displayed(*self._frame_locator)
        self.marionette.switch_to_frame(self.marionette.find_element(*self._frame_locator))

    def get_assertion(self):
        # Gets the last assertion in the event stream list, use logout event to make sure
        # we're done getting assertions
        return self.marionette.find_elements(*self._app_login_assertion_text)[-1].text

    def tap_standard_button(self):
        self.wait_for_element_displayed(*self._app_std_request_button_locator)
        self.marionette.find_element(*self._app_std_request_button_locator).tap()

    def tap_logout_button(self):
        self.wait_for_element_displayed(*self._app_logout_button_locator)
        self.marionette.find_element(*self._app_logout_button_locator).tap()

    def wait_for_logout_event(self):
        self.wait_for_element_displayed(*self._app_logout_event)

    def wait_for_ready_event(self):
        self.wait_for_element_displayed(*self._app_ready_event)

    def wait_for_login_event(self):
        self.wait_for_element_displayed(*self._app_login_event)



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import urllib2
import json
from gaiatest.mocks.mock_user import MockUser


class PersonaTestUser:
    """
    A base test class that can be extended by other tests to include utility methods.
    API docs: https://github.com/mozilla/personatestuser.org#api

    Usage:
    verified = bool:
    Verified refers to the user's account and password already approved and set up

    env = str:
    Strings "dev", "stage" or "prod" will return users for the respective environments
    If "None" a production Persona user will be returned.

    env = dict:
    For custom browserid databases and verifiers
    self.user = PersonaTestUser().create_user(verified=True,
        env={"browserid":"firefoxos.persona.org", "verifier":"firefoxos.123done.org"})

    """

    def create_user(self, verified=False, env=None):

        if verified:
            url = "http://personatestuser.org/email/"
        else:
            url = "http://personatestuser.org/unverified_email/"

        if type(env) is str:
            url += env

        elif type(env) is dict:
            url += "custom?"
            for index, i in enumerate(env):
                if index > 0:
                    url += "&"
                url += "%s=%s" % (i, env[i])

        try:
            # ptu.org will fail with a 400 if the parameters are invalid
            response = urllib2.urlopen(url)
        except urllib2.URLError as e:
            raise Exception("Could not get Persona user from personatestuser.org: %s" % e.reason)

        decode = json.loads(response.read())
        return MockUser(email=decode['email'], password=decode['pass'], name=decode['email'].split('@')[0])



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from marionette.marionette import Actions
from gaiatest.apps.base import Base


class Calendar(Base):

    name = 'Calendar'

    _current_month_year_locator = (By.ID, 'current-month-year')
    _current_months_day_locator = (By.ID, 'months-day-view')
    _current_monthly_calendar_locator = (By.ID, 'month-view')
    _hint_swipe_to_navigate_locator = (By.ID, 'hint-swipe-to-navigate')
    _add_event_button_locator = (By.XPATH, "//a[@href='/event/add/']")

    _week_display_button_locator = (By.XPATH, "//a[@href='/week/']")
    _day_display_button_locator = (By.XPATH, "//a[@href='/day/']")
    _day_view_locator = (By.ID, 'day-view')
    _week_view_locator = (By.ID, 'week-view')

    def launch(self):
        Base.launch(self)
        self.wait_for_element_displayed(*self._current_month_year_locator)
        self.wait_for_element_not_displayed(*self._hint_swipe_to_navigate_locator)

    @property
    def current_month_year(self):
        return self.marionette.find_element(*self._current_month_year_locator).text

    @property
    def current_month_day(self):
        return self.marionette.find_element(*self._current_months_day_locator).get_attribute('data-date')

    def tap_add_event_button(self):
        self.marionette.find_element(*self._add_event_button_locator).tap()

        from gaiatest.apps.calendar.regions.event import NewEvent
        new_event = NewEvent(self.marionette)
        new_event.wait_for_panel_to_load()
        return new_event

    def tap_week_display_button(self):
        self.marionette.find_element(*self._week_display_button_locator).tap()
        self.wait_for_element_displayed(*self._week_view_locator)

    def tap_day_display_button(self):
        self.marionette.find_element(*self._day_display_button_locator).tap()
        self.wait_for_element_displayed(*self._day_view_locator)

    def displayed_events_in_month_view(self, date_time):
        return self.marionette.find_element(*self._get_events_locator_in_month_view(date_time)).text

    def displayed_events_in_week_view(self, date_time):
        return self.marionette.find_element(*self._get_events_locator_in_week_view(date_time)).text

    def displayed_events_in_day_view(self, date_time):
        return self.marionette.find_element(*self._get_events_locator_in_day_view(date_time)).text

    def _get_events_locator_in_day_view(self, date_time):
        time_slot = self._get_data_hour(date_time)
        data_date = self._get_data_date(date_time)
        return (By.CSS_SELECTOR,
                "#day-view section.active[data-date*='%s'] section.hour-%d" % (data_date, time_slot))

    def _get_events_locator_in_month_view(self, date_time):
        time_slot = self._get_data_hour(date_time)
        return (By.CSS_SELECTOR,
                '#event-list section.hour-%d div.events' % time_slot)

    def _get_events_locator_in_week_view(self, date_time):
        time_slot = self._get_data_hour(date_time)
        data_date = self._get_data_date(date_time)
        return (By.CSS_SELECTOR,
                "#week-view section.active[data-date*='%s'] ol.hour-%d" % (data_date, time_slot))

    @staticmethod
    def _get_data_date(date_time):
        return date_time.strftime("%b %d")

    @staticmethod
    def _get_data_hour(date_time):
        return date_time.hour

    def flick_to_next_month(self):
        self._flick_to_month('next')

    def flick_to_previous_month(self):
        self._flick_to_month('previous')

    def _flick_to_month(self, direction):
        """Flick current monthly calendar to next or previous month.

        @param direction: flick to next month if direction='next', else flick to previous month
        """
        action = Actions(self.marionette)

        current_monthly_calendar = self.marionette.find_element(*self._current_monthly_calendar_locator)
        flick_origin_x = current_monthly_calendar.size['width'] // 2
        flick_origin_y = current_monthly_calendar.size['height'] // 2
        flick_destination_x = 0 if direction == 'next' else 2 * flick_origin_x

        action.flick(current_monthly_calendar, flick_origin_x, flick_origin_y,
                     flick_destination_x, flick_origin_y)
        action.perform()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.cost_control.app import CostControl


class FTUStep3(CostControl):

    _data_alert_header_locator = (By.CSS_SELECTOR, '#non-vivo-step-2 header')
    _ftu_usage_locator = (By.CSS_SELECTOR, '#non-vivo-step-2 span.tag')
    _ftu_data_alert_switch_locator = (By.CSS_SELECTOR, '#non-vivo-step-2 label.end input')
    _ftu_data_alert_label_locator = (By.CSS_SELECTOR, '#non-vivo-step-2 label.end')
    _unit_button_locator = (By.CSS_SELECTOR, '#data-limit-dialog form button span')
    _size_input_locator = (By.CSS_SELECTOR, '#data-limit-dialog form input')
    _usage_done_button_locator = (By.ID, 'data-usage-done-button')
    _go_button_locator = (By.CSS_SELECTOR, '#non-vivo-step-2 button.recommend')

    def __init__(self, marionette):
        CostControl.__init__(self, marionette)
        header = self.marionette.find_element(*self._data_alert_header_locator)
        self.wait_for_condition(lambda m: header.location['x'] == 0)

    def toggle_data_alert_switch(self, value):
        self.wait_for_element_displayed(*self._ftu_data_alert_label_locator)
        switch = self.marionette.find_element(*self._ftu_data_alert_switch_locator)
        if switch.is_selected() is not value:
            label = self.marionette.find_element(*self._ftu_data_alert_label_locator)
            label.tap()

    def select_when_use_is_above_unit_and_value(self, unit, value):
        self.wait_for_element_displayed(*self._ftu_usage_locator)
        usage = self.marionette.find_element(*self._ftu_usage_locator)
        usage.tap()

        self.wait_for_element_displayed(*self._unit_button_locator)
        current_unit = self.marionette.find_element(*self._unit_button_locator)
        # don't use "is not" here, they are different object
        if current_unit.text != unit:
            current_unit.tap()
            # We need to wait for the javascript to do its stuff
            self.wait_for_condition(lambda m: m.find_element(*self._unit_button_locator).text == unit)

        # clear the original assigned value and set it to the new value
        self.wait_for_element_displayed(*self._size_input_locator)
        size = self.marionette.find_element(*self._size_input_locator)
        size.clear()
        size.send_keys(value)
        done = self.marionette.find_element(*self._usage_done_button_locator)
        done.tap()

    def tap_lets_go(self):
        self.wait_for_element_displayed(*self._go_button_locator)
        self.marionette.find_element(*self._go_button_locator).tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import PageRegion


class PasscodePad(PageRegion):

    _numeric_button_locator = (By.CSS_SELECTOR, 'a[data-key="%s"]')
    _emergency_button_locator = (By.CSS_SELECTOR, 'a[data-key="e"]')

    def type_passcode(self, passcode):
        for digit in passcode:
            button_locator = (self._numeric_button_locator[0],
                              self._numeric_button_locator[1] % digit)
            self.root_element.find_element(*button_locator).tap()

    def tap_emergency_call(self):
        self.root_element.find_element(*self._emergency_button_locator).tap()

        from gaiatest.apps.system.regions.emergency_call import EmergencyCallScreen
        emergency_screen = EmergencyCallScreen(self.marionette)
        emergency_screen.switch_to_emergency_call_screen()

        return emergency_screen



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class Activities(Base):

    _actions_menu_locator = (By.CSS_SELECTOR, 'form[data-type="action"]')
    _action_option_locator = (By.CSS_SELECTOR, 'form[data-type="action"] button')

    _gallery_button_locator = (By.XPATH, '//*[text()="Gallery"]')
    _camera_button_locator = (By.XPATH, '//*[text()="Camera"]')
    _cancel_button_locator = (By.CSS_SELECTOR, 'form[data-type="action"] button[data-action="cancel"]')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.marionette.switch_to_frame()
        self.wait_for_element_displayed(*self._actions_menu_locator)

    def tap_gallery(self):
        self.marionette.find_element(*self._gallery_button_locator).tap()
        from gaiatest.apps.gallery.app import Gallery
        gallery = Gallery(self.marionette)
        gallery.switch_to_gallery_frame()
        return gallery

    def tap_camera(self):
        self.marionette.find_element(*self._camera_button_locator).tap()
        from gaiatest.apps.camera.app import Camera
        camera = Camera(self.marionette)
        camera.switch_to_camera_frame()
        return camera

    def tap_cancel(self):
        self.marionette.find_element(*self._cancel_button_locator).tap()
        self.marionette.switch_to_frame(self.apps.displayed_app.frame_id)

    @property
    def options_count(self):
        return len(self.marionette.find_elements(*self._action_option_locator))

    @property
    def is_menu_visible(self):
        return self.is_element_displayed(*self._actions_menu_locator)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class ContactDetails(Base):

    _contact_name_title_locator = (By.ID, 'contact-name-title')
    _contact_image_locator = (By.ID, 'cover-img')
    _call_phone_number_button_locator = (By.ID, 'call-or-pick-0')
    _send_sms_button_locator = (By.ID, 'send-sms-button-0')
    _edit_contact_button_locator = (By.ID, 'edit-contact-button')
    _back_button_locator = (By.ID, 'details-back')
    _add_remove_favorite_button_locator = (By.ID, 'toggle-favorite')
    _comments_locator = (By.ID, 'note-details-template-0')

    def __init__(self, marionette):
        Base.__init__(self, marionette)
        self.wait_for_condition(lambda m: m.find_element(*self._contact_name_title_locator).location['x'] == 0)

    @property
    def full_name(self):
        return self.marionette.find_element(*self._contact_name_title_locator).text

    @property
    def phone_number(self):
        return self.marionette.find_element(*self._call_phone_number_button_locator).text

    @property
    def comments(self):
        return self.marionette.find_element(*self._comments_locator).text

    @property
    def image_style(self):
        return self.marionette.find_element(*self._contact_image_locator).get_attribute('style')

    def tap_phone_number(self):
        self.marionette.find_element(*self._call_phone_number_button_locator).tap()
        from gaiatest.apps.phone.regions.call_screen import CallScreen
        return CallScreen(self.marionette)

    def tap_send_sms(self):
        self.marionette.find_element(*self._send_sms_button_locator).tap()
        from gaiatest.apps.messages.regions.new_message import NewMessage
        return NewMessage(self.marionette)

    def tap_edit(self):
        self.wait_for_element_displayed(*self._edit_contact_button_locator)
        self.marionette.find_element(*self._edit_contact_button_locator).tap()
        from gaiatest.apps.contacts.regions.contact_form import EditContact
        return EditContact(self.marionette)

    def tap_back(self):
        self.wait_for_element_displayed(*self._back_button_locator)
        self.marionette.find_element(*self._back_button_locator).tap()
        self.wait_for_element_not_displayed(*self._back_button_locator)
        from gaiatest.apps.contacts.app import Contacts
        return Contacts(self.marionette)

    def tap_add_remove_favorite(self):
        button = self.marionette.find_element(*self._add_remove_favorite_button_locator)
        self.marionette.execute_script("arguments[0].scrollIntoView(false);", [button])
        button.tap()

    @property
    def add_remove_text(self):
        return self.marionette.find_element(*self._add_remove_favorite_button_locator).text



import argparse
import sys

from marionette import Marionette

import gaiatest


class GCli(object):

    def __init__(self):
        self.commands = {
            'connectwifi': {
                'function': self.connect_to_wifi,
                'args': [
                    {'name': 'ssid',
                     'help': 'SSID of the network to connect to'},
                    {'name': '--security',
                     'choices': ['WPA-PSK', 'WEP'],
                     'help': 'Security model of the network'},
                    {'name': '--password',
                     'help': 'Password to access the network'}],
                'help': 'Connect to a WiFi network'},
            'disablewifi': {
                'function': self.disable_wifi,
                'help': 'Disable WiFi'},
            'enablewifi': {
                'function': self.enable_wifi,
                'help': 'Enable WiFi'},
            'forgetallwifi': {
                'function': self.forget_all_wifi_networks,
                'help': 'Forget all WiFi networks'},
            'getknownwifi': {
                'function': self.known_wifi_networks,
                'help': 'Show known WiFi networks'},
            'getsetting': {
                'function': self.get_setting,
                'args': [{
                    'name': 'name',
                    'help': 'Name of the setting to retrieve'}],
                'help': 'Show the current value of a setting'},
            'holdhome': {
                'function': self.hold_home,
                'help': 'Simulate holding the home button'},
            'holdsleep': {
                'function': self.hold_sleep,
                'help': 'Simulate holding the sleep button'},
            'home': {
                'function': self.home,
                'help': 'Simulate pressing the home button'},
            'killapps': {
                'function': self.kill_all_apps,
                'help': 'Kill all running apps'},
            'launchapp': {
                'function': self.launch_app,
                'args': [
                    {'name': 'name',
                     'nargs': argparse.REMAINDER,
                     'help': 'Name of app to launch'}],
                'help': 'Launch an application'},
            'listallapps': {
                'function': self.list_all_apps,
                'help': 'List all apps'},
            'listrunningapps': {
                'function': self.list_running_apps,
                'help': 'List the running apps'},
            'lock': {
                'function': self.lock,
                'help': 'Lock screen'},
            'screenshot': {
                'function': self.screenshot,
                'help': 'Take a screenshot'},
            'sendsms': {
                'function': self.send_sms,
                'args': [
                    {'name': 'number',
                     'help': 'Phone number of recipient'},
                    {'name': 'message',
                     'help': 'Message content'}],
                'help': 'Send an SMS'},
            'setsetting': {
                'function': self.set_setting,
                'args': [
                    {'name': 'name',
                     'help': 'Name of setting to change'},
                    {'name': 'value',
                     'help': 'New value for setting'}],
                'help': 'Change the value of a setting'},
            'sleep': {
                'function': self.sleep,
                'help': 'Enter sleep mode'},
            'unlock': {
                'function': self.unlock,
                'help': 'Unlock screen'},
            'volume': {
                'function': self.volume,
                'args': [
                    {'name': 'direction',
                     'choices': ['down', 'up'],
                     'help': 'Direction to change the volume'}],
                'help': 'Change the volume'},
            'wake': {
                'function': self.wake,
                'help': 'Wake from sleep mode'}}

        self.parser = argparse.ArgumentParser()
        self.add_options(self.parser)
        self.add_commands(self.parser)

    def run(self, args=sys.argv[1:]):
        args = self.parser.parse_args()

        host, port = args.address.split(':')
        self.marionette = Marionette(host=host, port=int(port))
        self.marionette.start_session()

        self.apps = gaiatest.GaiaApps(self.marionette)
        self.data_layer = gaiatest.GaiaData(self.marionette)
        self.lock_screen = gaiatest.LockScreen(self.marionette)

        ret = args.func(args)
        if ret is None:
            ret = 0

        self.marionette.delete_session()

        sys.exit(ret)

    def add_options(self, parser):
        parser.add_argument(
            '--address',
            default='localhost:2828',
            help='Address (host:port) of running Gecko instance to connect to '
                 '(default: %(default)s)')

    def add_commands(self, parser):
        subparsers = parser.add_subparsers(
            title='Commands', metavar='<command>')
        for (name, props) in sorted(self.commands.iteritems()):
            subparser = subparsers.add_parser(name, help=props['help'])
            if props.get('args'):
                for arg in props['args']:
                    kwargs = {k: v for k, v in arg.items() if k is not 'name'}
                    subparser.add_argument(arg['name'], **kwargs)
            subparser.set_defaults(func=props['function'])

    def connect_to_wifi(self, args):
        network = {
            'ssid': args.ssid,
            'keyManagement': args.security or 'NONE'}
        if args.security == 'WEP':
            network['wep'] = args.password
        elif args.security == 'WPA-PSK':
            network['psk'] = args.password
        self.data_layer.connect_to_wifi(network)

    def disable_wifi(self, args):
        self.data_layer.disable_wifi()

    def enable_wifi(self, args):
        self.data_layer.enable_wifi()

    def forget_all_wifi_networks(self, args):
        self.data_layer.forget_all_networks()

    def get_setting(self, args):
        print '%s: %s' % (
            args.name,
            self.data_layer.get_setting(args.name))

    def home(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('home'));")

    def hold_home(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('holdhome'));")

    def hold_sleep(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('holdsleep'));")

    def kill_all_apps(self, args):
        self.apps.kill_all()

    def known_wifi_networks(self, args):
        networks = [n for n in self.data_layer.known_networks if 'ssid' in n]
        if len(networks) > 0:
            for i, network in enumerate(networks):
                print '%s: %s' % (i + 1, network['ssid'])
        else:
            print 'No known networks.'

    def launch_app(self, args):
        for name in args.name:
            self.apps.launch(name)

    def list_all_apps(self, args):
        for i, app in enumerate(sorted(self.apps.installed_apps,
                                       key=lambda a: a.name.lower())):
            print '%d: %s' % (i + 1, app.name)

    def list_running_apps(self, args):
        for i, app in enumerate(sorted(self.apps.running_apps,
                                       key=lambda a: a.name.lower())):
            print '%d: %s' % (i + 1, app.name)

    def lock(self, args):
        self.lock_screen.lock()

    def screenshot(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('home+sleep'));")

    def send_sms(self, args):
        self.data_layer.send_sms(args.number, args.message)

    def set_setting(self, args):
        self.data_layer.set_setting(args.name, args.value)

    def sleep(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('sleep'));")

    def unlock(self, args):
        self.lock_screen.unlock()

    def volume(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('volume%s'));" %
            args.direction)

    def wake(self, args):
        self.marionette.execute_script(
            "window.wrappedJSObject.dispatchEvent(new Event('wake'));")


def cli(args=sys.argv[1:]):
    cli = GCli()
    cli.run(args)

if __name__ == '__main__':
    cli()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class ContactForm(Base):

    _given_name_locator = (By.ID, 'givenName')
    _family_name_locator = (By.ID, 'familyName')
    _phone_locator = (By.ID, 'number_0')
    _email_locator = (By.ID, 'email_0')
    _street_locator = (By.ID, 'streetAddress_0')
    _zip_code_locator = (By.ID, 'postalCode_0')
    _city_locator = (By.ID, 'locality_0')
    _country_locator = (By.ID, 'countryName_0')
    _comment_locator = (By.ID, 'note_0')

    _thumbnail_photo_locator = (By.ID, 'thumbnail-photo')

    @property
    def given_name(self):
        return self.marionette.find_element(*self._given_name_locator).text

    def type_given_name(self, value):
        element = self.marionette.find_element(*self._given_name_locator)
        element.clear()
        element.send_keys(value)

    @property
    def family_name(self):
        return self.marionette.find_element(*self._family_name_locator).text

    def type_family_name(self, value):
        element = self.marionette.find_element(*self._family_name_locator)
        element.clear()
        element.send_keys(value)

    @property
    def phone(self):
        return self.marionette.find_element(*self._phone_locator).text

    def type_phone(self, value):
        element = self.marionette.find_element(*self._phone_locator)
        element.clear()
        element.send_keys(value)

    @property
    def email(self):
        return self.marionette.find_element(*self._email_locator).text

    def type_email(self, value):
        element = self.marionette.find_element(*self._email_locator)
        element.clear()
        element.send_keys(value)

    @property
    def street(self):
        return self.marionette.find_element(*self._street_locator).text

    def type_street(self, value):
        element = self.marionette.find_element(*self._street_locator)
        element.clear()
        element.send_keys(value)

    @property
    def zip_code(self):
        return self.marionette.find_element(*self._zip_code_locator).text

    def type_zip_code(self, value):
        element = self.marionette.find_element(*self._zip_code_locator)
        element.clear()
        element.send_keys(value)

    @property
    def city(self):
        return self.marionette.find_element(*self._city_locator).text

    def type_city(self, value):
        element = self.marionette.find_element(*self._city_locator)
        element.clear()
        element.send_keys(value)

    @property
    def country(self):
        return self.marionette.find_element(*self._country_locator).text

    def type_country(self, value):
        element = self.marionette.find_element(*self._country_locator)
        element.clear()
        element.send_keys(value)

    @property
    def comment(self):
        return self.marionette.find_element(*self._comment_locator).text

    def type_comment(self, value):
        self.wait_for_element_displayed(*self._comment_locator)
        element = self.marionette.find_element(*self._comment_locator)
        element.clear()
        element.send_keys(value)

    @property
    def picture_style(self):
        return self.marionette.find_element(*self._thumbnail_photo_locator ).get_attribute('style')

    def tap_picture(self):
        self.marionette.find_element(*self._thumbnail_photo_locator).tap()
        from gaiatest.apps.system.regions.activities import Activities
        return Activities(self.marionette)

    def wait_for_image_to_load(self):
        self.wait_for_condition(lambda m: 'background-image' in self.picture_style)

class EditContact(ContactForm):

    _update_locator = (By.ID, 'save-button')
    _cancel_locator = (By.ID, 'cancel-edit')
    _delete_locator = (By.ID, 'delete-contact')
    _delete_form_locator = (By.ID, 'confirmation-message')
    _cancel_delete_locator = (By.CSS_SELECTOR, 'form#confirmation-message button:not(.danger)')
    _confirm_delete_locator = (By.CSS_SELECTOR, 'form#confirmation-message button.danger')

    def __init__(self, marionette):
        ContactForm.__init__(self, marionette)
        update = self.marionette.find_element(*self._update_locator)
        self.wait_for_condition(lambda m: update.location['y'] == 0)

    def tap_update(self):
        self.wait_for_update_button_enabled()
        self.marionette.find_element(*self._update_locator).tap()
        self.wait_for_element_not_displayed(*self._update_locator)
        from gaiatest.apps.contacts.regions.contact_details import ContactDetails
        return ContactDetails(self.marionette)

    def tap_cancel(self):
        self.marionette.find_element(*self._cancel_locator).tap()
        from gaiatest.apps.contacts.regions.contact_details import ContactDetails
        return ContactDetails(self.marionette)

    def tap_delete(self):
        delete_item = self.marionette.find_element(*self._delete_locator)
        # TODO Bug 875830 - Remove scrollIntoView() when bug resolved
        self.marionette.execute_script("arguments[0].scrollIntoView(false);", [delete_item])
        delete_item.tap()

    def tap_cancel_delete(self):
        self.wait_for_element_displayed(*self._delete_form_locator)
        self.marionette.find_element(*self._cancel_delete_locator).tap()

    def tap_confirm_delete(self):
        self.wait_for_element_displayed(*self._delete_form_locator)
        self.marionette.find_element(*self._confirm_delete_locator).tap()
        from gaiatest.apps.contacts.app import Contacts
        return Contacts(self.marionette)

    def wait_for_update_button_enabled(self):
        self.wait_for_condition(lambda m: self.marionette.find_element(*self._update_locator).is_enabled())


class NewContact(ContactForm):

    _done_button_locator = (By.ID, 'save-button')

    def __init__(self, marionette):
        ContactForm.__init__(self, marionette)
        done = self.marionette.find_element(*self._done_button_locator)
        self.wait_for_condition(lambda m: done.location['y'] == 0)

    def tap_done(self):
        self.marionette.find_element(*self._done_button_locator).tap()
        self.wait_for_element_not_displayed(*self._done_button_locator)
        from gaiatest.apps.contacts.app import Contacts
        return Contacts(self.marionette)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By

from gaiatest.apps.base import Base


class AuthenticationDialog(Base):

    _auth_dialog_locator = (By.ID, 'http-authentication-dialog')

    _username_input_locator = (By.ID, 'http-authentication-username')
    _password_input_locator = (By.ID, 'http-authentication-password')
    _sign_in_button_locator = (By.ID, 'http-authentication-ok')

    def type_username(self, text):
        username_input = self.marionette.find_element(*self._username_input_locator)
        username_input.send_keys(text)

    def type_password(self, text):
        username_input = self.marionette.find_element(*self._password_input_locator)
        username_input.send_keys(text)

    def authenticate(self, username, password):
        self.wait_for_element_displayed(*self._auth_dialog_locator)
        self.type_username(username)
        self.type_password(password)

        sign_in_button = self.marionette.find_element(*self._sign_in_button_locator)
        sign_in_button.tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from marionette.keys import Keys
from marionette.marionette import Actions

from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class SearchPanel(Base):

    _body = (By.CSS_SELECTOR, 'body')
    _search_title_type_locator = (By.CSS_SELECTOR, '#search-title > .type')
    _search_title_query_locator = (By.CSS_SELECTOR, '#search-title > .query')
    _search_title_first_word_locator = (By.CSS_SELECTOR, '#search-title [data-l10n-id="evme-helper-title-prefix"]')
    _search_results_from_everything_me_locator = (By.CSS_SELECTOR, '#search .evme-apps ul.cloud li[data-name]')
    _search_results_installed_app_locator = (By.CSS_SELECTOR, '#search .evme-apps ul.installed li[data-name]')
    _app_icon_locator = (By.CSS_SELECTOR, 'ul.cloud li[data-name]')

    def type_into_search_box(self, search_term):
        self.keyboard.send(search_term)
        self.keyboard.tap_enter()

        self.wait_for_condition(lambda m: self.marionette.find_element(*self._search_title_query_locator).text.lower() ==
                                search_term.lower())

        self.wait_for_element_displayed(*self._search_title_first_word_locator)

    def wait_for_everything_me_loaded(self):
        self.wait_for_condition(
            lambda m: 'evme-loading' not in self.marionette.find_element(*self._body).get_attribute('class'))

    def wait_for_everything_me_results_to_load(self):
        self.wait_for_element_displayed(*self._search_results_from_everything_me_locator)

    def wait_for_type(self, type):
        self.wait_for_condition(lambda m: type.lower() in self.marionette.find_element(*self._search_title_type_locator).text.lower())

    def wait_for_app_icons_displayed(self):
        self.wait_for_element_displayed(*self._app_icon_locator)

    def wait_for_installed_apps_displayed(self):
        self.wait_for_element_displayed(*self._search_results_installed_app_locator)

    @property
    def results(self):
        return [self.Result(marionette=self.marionette, element=result)
                for result in self.marionette.find_elements(*self._search_results_from_everything_me_locator)]

    @property
    def installed_apps(self):
        return [self.InstalledApp(self.marionette, root_el) for root_el in
                self.marionette.find_elements(*self._search_results_installed_app_locator)]

    class Result(PageRegion):

        @property
        def name(self):
            return self.root_element.get_attribute('data-name')

        def tap(self):
            app_name = self.name

            self.root_element.tap()
            # Wait for the displayed app to be that we have tapped
            self.wait_for_condition(lambda m: self.apps.displayed_app.name == app_name)
            self.marionette.switch_to_frame(self.apps.displayed_app.frame)

            # Wait for title to load (we cannot be more specific because the aut may change)
            self.wait_for_condition(lambda m: m.title)

    class InstalledApp(PageRegion):

        @property
        def name(self):
            return self.root_element.get_attribute('data-name')

        def tap(self):
            expected_name = self.name
            self.root_element.tap()
            self.wait_for_condition(
                lambda m: self.apps.displayed_app.name.lower() == expected_name.lower())
            self.marionette.switch_to_frame(self.apps.displayed_app.frame)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base


class PermissionDialog(Base):

    _permission_dialog_locator = (By.ID, 'permission-dialog')
    _permission_dialog_message_locator = (By.ID, 'permission-message')
    _permission_confirm_button_locator = (By.ID, 'permission-yes')
    _permission_dismiss_button_locator = (By.ID, 'permission-no')

    def wait_for_permission_dialog_displayed(self):
        self.wait_for_element_displayed(*self._permission_dialog_locator)

    @property
    def permission_dialog_message(self):
        return self.marionette.find_element(*self._permission_dialog_message_locator).text

    def tap_to_confirm_permission(self):
        self.marionette.find_element(*self._permission_confirm_button_locator).tap()

    def tap_to_dismiss_permission(self):
        self.marionette.find_element(*self._permission_dismiss_button_locator).tap()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from marionette.by import By
from gaiatest.apps.base import Base
from gaiatest.apps.base import PageRegion


class UtilityTray(Base):
    _notification_container_locator = (By.ID, 'notifications-container')
    _desktop_notifications_locator = (By.CSS_SELECTOR, '#desktop-notifications-container .notification')
    _notification_clear_locator = (By.ID, 'notification-clear')
    _quicksettings_app_locator = (By.ID, 'quick-settings-full-app')

    def wait_for_notification_container_displayed(self):
        # Marionette cannot read the displayed state of the notification container so we wait for its location
        self.wait_for_condition(lambda m: m.find_element(*self._notification_container_locator).location['y'] == 50)

    @property
    def notifications(self):
        return [Notification(self.marionette, notification)
                for notification in self.marionette.find_elements(*self._desktop_notifications_locator)]

    def clear_all_notifications(self):
        self.marionette.find_element(*self._notification_clear_locator).tap()

    def tap_settings_button(self):
        self.marionette.find_element(*self._quicksettings_app_locator).tap()

class Notification(PageRegion):
    _body_locator = (By.CSS_SELECTOR, 'div.detail')

    @property
    def content(self):
        return self.root_element.find_element(*self._body_locator).text
